c
c             This program plots diffraction spots on spedified
c             diffraction planes and outputs Postscript (.ps) files
c
c     (01) file name for Fo file
c     (02) file name for Fo temp file
c     (03) file name for Fc temp file
c
c     (1) title 
c
c     (2) a b c cosa cosb cosc
c
c     (3) nsub zmtrx
c         nsub: number of subsystems (for composite crystal nsub>=2)
c         if(nsub>=2) then zmtrx
c     (4) ndm,nsymo,icent,ige,nctr
c         ndm: dimension of modulation     (1<=ndm<=3)
c         nsymo: number of symmetry operators in modulated structure
c                 (1<=nsymo<=48)
c         icent=0: noncentrosymmetric, =1: centrosymmetric
c           if centrosymmetric take the origin at center of symmetry
c           and give symmetry operators in the factor group, g/i
c           ige=0: give all symmetry opeators, ige=1: give generators,
c           then all elements are created in the program
c         nctr: number of centering translations
c
c     (5)  a*,b*,c* components of the wave vector icorc, ilm (*)
c         if incommensurate icorc=1, commensurate icorc=0
c         ilm  number of divisions of gauss integrals in structure
c         factor formula in incommensurate cases     3<=ilm<=100
c           =========== repeat (4) ndm times ============
c
c           if all the wave vectors are incommensurate
c                          skip (6)
c **************data for the commensurate structure*********************
c *       if anyone of wave vectors is commensurate,                   *
c *   (6)  give period along a,b,c axes (ilc)                          *
c **********************************************************************
c      *********if nctr=0 then skip (6) ***********
c     (7) centering translation
c           in the format of I.T. for example 1/2,1/2,0,1/2 etc.
c           ====== repeat (7) nctr times =======
c     (8) symmetry operatores
c           in the format of I.T. for example x,y,z,t,u,v etc.
c           ======== repeat (8) nsymo times ==========
c         ****the first operator must be the identity operator****
c         ****the symmetry operator is used to classify the
c                            satellite reflections    see (8)******
c
c     (9) nds : number of data sets
c
c     (10) reflection selection variables ng
c          ng: number of reflection groups
c              =0 all reflections are selected in the refinement
c                 if(ng.0) h3+i
c                 this is used to remove specified reflections
c
c     (11) fmin,wg0,hm
c
c     (12) inp,iorf,format
c          inp=5 (standard input), or inp=3 (specified file)
c          iorf=0,1 iobs and fobs
c          format for indices and fo or fc and wght
c
c                        if inp=3  skip (13)
c***********************************************************************
c*    (13) h1,h2.. fobs (or iobs), wght                                *
c*         miller indices fobs (or iobs) weight according to the format*
c*         specified above                                             *
c*                repeat (40) up to the last reflection                *
c*            ****h1=999 mean the end of each data sets****            *
c***********************************************************************
c
c     (14) fsig
c          fsig : fo or fc > fsig*wght are plotted
c     (15) iselect
c          iselect=0 : all the reflections are ploted
c          iselect=1 : reflections violating the reflection conditions
c                      are removed
c     (16) nsh, ldev
c           nsh: number of sheets
c           ldev: 1 B/W, 2 color printer
c                reflections in this range will be projected
c     (17) ng : number of graphs in a sheet
c     (18)  icase,hmin,hmax,amag,rdiff,shiftx,shifty
c           icase=1  plot a*, b* plane with l=ifix
c           icase=2  plot b*, c* plane with h=ifix
c           icase=3  plot c*, a* plane with k=ifix
c           icase=4  plot (h,h,l) plane with h-k=ifix
c                for tetragonal, hexagonal and cubic system
c           icase=5  specified plane
c           hmin, hmax : range of ifix (real number)
c               (projection of several planes is possible)
c           amag : magnification of the figure
c           rdiff : scale factor for diffraction spots
c           shiftx, shifty: shifts along x, y coordinate in the
c                           plotter system  (in cm)
c          ******if icase is less than 5 skip (19,20)*****
c     (19) a3, 3rd axis (normal to a reciprocal plane plotted)
c     (20) a2, 2nd axis (y-axis)
c           a1, 1st axis (x-axis) is calculated from a2 and a3
c          *********************************************
c     (21)  title, x, y, h
c           title: title of a figure
c           x,y: coordinates of the title (in cm)
c           h  : hight of characters used (in cm)
c    ************  repeat (18-20) ng times*******
c    ************  repeat (17-20) nsh times *****
c
      parameter (nhkl=3000)
      character*80 flnm,flnm3,flnm4
      common /ccol/ icol
      common /ccase/ icase, ifix, hmin, hmax, a1(3), a2(3), a3(3)
      common /cwflag/ iwflag(6)
      common /bgn/ ilm(3),icorc(3),ilc(3)
      common /cplt/ amag,rdiff,shiftx,shifty
      integer hd(6,nhkl),hr(6)
      real    fo(nhkl),fc(nhkl)
      integer hkln
      complex fcal
      common /fhkl/ hkln(100),fobs(100),fcal(100),wght(100),im(20),is(2)
     *         ,sf
      character*26 flfobs,flfcal
      common /cnsti/ ix,ib,if,itvr,itst,nds,na4
      common /filen/ flfobs,flfcal
      common /todrw/ iselect
      character*80 wtitle
      integer ih(6)
      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      data iwflag/1,1,1,1,1,1/
      data zmtrx /1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6/
      data nsub/1/
c
c     initialize
c
      icorc(1)=0
      icorc(2)=0
      icorc(3)=0
      pcol=1
      icol=10
c
      read(5,'(a)') flnm
      read(5,'(a)') flnm3
      read(5,'(a)') flnm4
      flfobs=flnm3
      call setinp(5)
      call setout(6)
      call rdlc
      call freadi('nsub=',1,nsub,1)
      if(nsub.le.0.or.nsub.gt.3) call quit('*** nsub invalid***')
      if(nsub.ge.2) then
          call freadi('zmtrx=',6,zmtrx(1,2),0)
      end if
      if(nsub.eq.3) then
          call freadi('zmtrx=',6,zmtrx(1,3),0)
      end if
      if(iwflag(3).eq.1) then
           write(6,*) 'nsub=',nsub,
     *        ('zmtrx=',(zmtrx(i,n),i=1,6),n=2,nsub)
      end if
      call rdsymo(na,isk)
      call getgij(zmtrx,nsub)
      call freadi('nds=',1,nds,1)
      write(6,*) 'nds=',nds
      iwght=1
      open(unit=3,file=flnm)
      open(unit=91,file=flnm3,access='direct',recl=1200)
      open(unit=92,file=flnm4,access='direct',recl=800)
      call rdiobs(iwght)
      call freadr('fsig=',1,fsig,1)
      call freadi('iselect=',1,iselect,1)
      write(6,*) 'iselect=',iselect, ' ldev=',ldev
      call freadi('nsh=',1,nsh,1)
      call freadi('ldev=',1,ldev,0)
      write(6,*) 'nsh=',nsh
      call setnfil(nsh)
      do nsd=1,nsh
         call freadi('ng=',1,ng,1)
         write(6,*) 'ng=',ng
         call plots(0.0,0.0,ldev)
         call newpen(1)
         shx=0
         shy=0
         do nn=1,ng
            call freadi('icase=',1,icase,1)
            call freadr('hmin=',1,hmin,0)
            call freadr('hmax=',1,hmax,0)
            call freadr('amag=',1,amag,0)
            call freadr('rdiff=',1,rdiff,0)
            call freadr('shiftx=',1,shiftx,0)
            call freadr('shifty=',1,shifty,0)
            write(6,*) 'icase=',icase,' hmin=',hmin,' hmax=',hmax
            write(6,*) 'amag=',amag,
     *           ' rdiff=',rdiff,' shiftx=',shiftx,' shifty=',shifty
            if(icase.eq.5) then
               call freadr('a3=',3,a3,1)
               call freadr('a2=',3,a2,1)
               write(6,*) 'a3=',a3
               write(6,*) 'a2=',a2
               call yyy(a3)
               call yyy(a2)
               call norml(a3,3)
               call norml(a2,3)
               a1(1)=a2(2)*a3(3)-a2(3)*a3(2)
               a1(2)=a2(3)*a3(1)-a2(1)*a3(3)
               a1(3)=a2(1)*a3(2)-a2(2)*a3(1)
            end if
            shx=shiftx
            shy=shifty
            call freadc('title=',1,wtitle,1)
            call freadr('x=',1,wx,0)
            call freadr('y=',1,wy,0)
            call freadr('h=',1,wh,0)
            wx=wx*amag
            wy=wy*amag
            wh=wh*amag
            do 80 i=0,79
               j=80-i
               if(wtitle(j:j).ne.' ') go to 81
 80         continue
 81         ntitle=j
            call plot(shx*amag,shy*amag,-3)
            call symbol(wx,wy,wh,wtitle,angl,ntitle)
            do 11 inds=1,nds
               ime=im(inds)
               if(ime.eq.0) go to 11
               if(inds.eq.1) ib=0
               ibe=0
               if(inds.gt.1) then
                  inds1=inds-1
                  do 343 ijk=1,inds1
                     ibe=ibe+im(ijk)
 343              continue
                  ib=mod(ibe,100)
                  ibe=ibe/100
                  if(ib.gt.0) then
                     ibe=ibe+1
                     read(unit=91,rec=ibe) hkln,fobs,wght
c                    read(unit=92,rec=ir1) fcal
                  end if
               end if
               iba=0
               do 10 ia=1,ime
                  ib=ib+1
                  ib=mod(ib-1,100)+1
                  if(ib.eq.1) then
                     iba=iba+1
                     read(unit=91,rec=ibe+iba) hkln,fobs,wght
c     read(unit=92,rec=ir1) fcal
                  end if
                  call ihkln(ih,hkln(ib))
c     call pltfoc(job)
c     call dfpttn(brv)
                  if(fobs(ib).gt.fsig*wght(ib)) then
                     call drwedf(ih,fobs(ib))
                  end if
 10            continue
 11         continue
         end do
         call plote('end of plot')
      end do
      close(unit=3)
      close(unit=91)
      close(unit=92)
      stop
      end

      subroutine extncr(ih,jskip)
      integer ih(*)
      common /cctr/ ctr(6,0:48),nctr
      common /cdim/ idm,nde,ied
      data pi2/6.28318/, eps/0.0001/
      jskip=0
      if(nctr.eq.0) return
      fctr=1
      fcti=0
      do 10 i=1,nctr
      fm=0
      do 11 j=1,idm
         fm=fm+ih(j)*ctr(j,i)
 11   continue
      fctr=fctr+cos(pi2*fm)
      fcti=fcti+sin(pi2*fm)
 10   continue
      if(abs(fctr).lt.eps.and.abs(fcti).lt.eps) then
          jskip=1
      else
          jskip=0
      end if
      return
      end

      subroutine dfpttn(brv)
      character brv
      common /cldev/ ldev,icol1,icol2,icase
      real yw(3),q(6)
      parameter (nhkl=3000)
      integer hd(6,nhkl),hr(6)
      real    fo(nhkl),fc(nhkl),q1(6),q2(6),q3(6)
      integer n1(6),n3(6)
      character*60 frm
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      data eps1/0.0001/
      data n3 /1,1,1,0,0,0/
      data n1 /0,0,0,-1,-1,0/
c     initialize
c      call getqi0(n3,q3)
c      call getqi0(n1,q1)
c      call norml(q3,3)
c      call norml(q1,3)
      q2(1)=q3(2)*q1(3)-q3(3)*q1(2)
      q2(2)=q3(3)*q1(1)-q3(1)*q1(3)
      q2(3)=q3(1)*q1(2)-q3(2)*q1(1)
      call freadc('format=',1,frm,1)
      write(6,*) 'format=',frm
      fact=1.
      n=0
      fomax=0
 70   continue
      n=n+1
      read(5,frm,end=71) (hd(j,n),j=1,6),fo(n),fc(n)
      write(6,frm)  (hd(j,n),j=1,6),fo(n),fc(n)
      if(fomax.lt.fo(n)) fomax=fo(n)
      if(hd(1,n).ne.999) go to 70
 71   continue
      if(hd(1,n).eq.999) n=n-1
      if(icase.eq.0) then
          ls=1
          le=3
      else if(icase.gt.0) then
          ls=icase
          le=icase
      end if
c      call getsys(isys)
      do 80 icol=1,2
          do 79 ll=ls,le
          x0=1.5
          y0=1.5
          x1=1.8
          y1=1.8
          x00=(x0+9*(icol-1))/29.7
          y00=(y0+9*(ll-1))/29.7
          x01=(x0+8.5+9*(icol-1))/29.7
          y01=(y0+8.5+9*(ll-1))/29.7
          call window(-x1,-y1,x1,y1)
          call viewpt(x00,y00,x01,y01)
          do 75 l=1,n
               do 751 ic=0,1
               do 751 ns=1,nsymo
               if(icol.eq.1) then
                     rc=fo(l)/(fomax*10)
               else
                     rc=fc(l)/(fomax*10)
               end if
               call gethr(hd(1,l),hr,rr,ns,ic)
c               call getqi0(hr,q)
               if(ll.eq.1) then
                     if(abs(q(3)).le.eps1) then
c                       z-axis (five or tenfold axis)
                           istl=isatel(hr,brv)
                           if(istl.eq.0) then
                               call fillcolor(icol1)
                           else
                               call fillcolor(icol2)
                           end if
                           yw(1)=q(1)
                           yw(2)=q(2)
                           call circle(0.,360.,rc,yw,0)
                      end if
                else if(ll.eq.2) then
c                       y-axis
                       if(abs(q(2)).le.eps1) then
                           istl=isatel(hr,brv)
                           if(istl.eq.0) then
                               call fillcolor(icol1)
                           else
                               call fillcolor(icol2)
                           end if
                           yw(1)=q(1)
                           yw(2)=q(3)
                           call circle(0.,360.,rc,yw,0)
                       end if
                else if(ll.eq.3) then
                    if(isys.le.2) then
c                      three-fold axis
                       if(abs(vinp(q,q3,3)).le.eps1) then
                           istl=isatel(hr,brv)
                           if(istl.eq.0) then
                               call fillcolor(icol1)
                           else
                               call fillcolor(icol2)
                           end if
                           yw(1)=vinp(q,q1,3)
                           yw(2)=vinp(q,q2,3)
                           call circle(0.,360.,rc,yw,0)
                       end if
                    else
c                      x-axis
                       if(abs(q(1)).le.eps1) then
                           istl=isatel(hr,brv)
                           if(istl.eq.0) then
                               call fillcolor(icol1)
                           else
                               call fillcolor(icol2)
                           end if
                           yw(1)=q(1)
                           yw(2)=q(2)
                           call circle(0.,360.,rc,yw,0)
                       end if
                    end if
                end if
 751            continue
 75       continue
 79   continue
 80   continue
      return
      end

      subroutine pltfoc(job)
      parameter (nhkl=3000)
      real xw(nhkl),yw(nhkl),zw(nhkl)
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qd(6)
      character*60 frm
      common /cldev/ ldev,icol1,icol2,icase
      call fillcolor(0)
      call freadc('format=',1,frm,1)
      write(6,*) 'format=',frm
      fomax=0
      qimax=0
      nref=0
      n=0
 100  continue
          n=n+1
          read(5,frm,end=200) (hd(j,n),j=1,6),fo(n),fc(n)
          if(hd(1,n).eq.999) go to 200
              if(fomax.lt.fo(n)) fomax=fo(n)
              write(6,frm)  (hd(j,n),j=1,6),fo(n),fc(n)
              nref=nref+1
          go to 100
 200  continue
      x0=1.
      y0=1.
      x1=15.
      x00=(x0)/29.7
      y00=(y0)/29.7
      x01=(x0+16)/29.7
      y01=(y0+16)/29.7
      call window(-x0,-x0,x1+x0,x1+x0)
      call viewpt(x00,y00,x01,y01)
      call newpen(2)
c       write box
      call plot(0.,0.,3)
      call plot(x1,0.,2)
      call plot(x1,x1,2)
      call plot(0.,x1,2)
      call plot(0.,0.,2)
      nmx=0
      do 400 n=1,nref
          nmx=nmx+1
          if(job.eq.2) then
              xw(nmx)=fo(n)*14.5/fomax
              yw(nmx)=fc(n)*14.5/fomax
          else if(job.eq.3) then
              yw(nmx)=fo(n)*14.5/fomax
              zw(nmx)=fc(n)*14.5/fomax
              xw(nmx)=qi(n)*14.5/qimax
          end if
c         go to 300
 400  continue
      xw(nmx+1)=0.
      yw(nmx+1)=0.
      zw(nmx+1)=0.
      xw(nmx+2)=1.0
      yw(nmx+2)=1.0
      zw(nmx+2)=1.0
c     plot normalized observed intensity
      call fillcolor(icol1)
      call line(xw,yw,nmx,1,-1,3)
      call fillcolor(icol2)
      call line(xw,zw,nmx,1,-1,2)
      return
      end

      subroutine drwedf(h,fc)
      integer h(6)
      real fc

      integer hr(6),hold(6,120)
      real eps,xc(3),xold(2,120)
      integer rno
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      common /cwflag/ iwflag(6)

      common /cplt/ amag,rdiff,shiftx,shifty
      common /ccol/ icol
      common /todrw/ iselect
      data eps/0.0001/

      rc=0.1*amag/5
 100  continue
      rno=0
           ndiff=0
           do 160 icnt=0,1
           do 160 nsy=1,nsymo
           call gethr(h,hr,rr,nsy,icnt)
           if(icnt.eq.1) then
               do 1610 id=1,6
               hr(id)=-hr(id)
 1610          continue
           end if
           call extncr(hr,jskip)
           if(iselect.eq.1.and.jskip.eq.1) go to 160
           if(jslct(hr).eq.0) go to 160
           call trcs(hr,xc)
c     select independent diffraction spots
           do 164 nd=1,ndiff
                if(abs(xc(1)-xold(1,nd)).lt.eps.and.
     *             abs(xc(2)-xold(2,nd)).lt.eps) then
                   go to 165
                end if
 164       continue
           ndiff=ndiff+1
           call savexc(xc,xold,ndiff)
           call savehr(hr,hold,ndiff)
 165       continue
 160       continue
c          plot equivalent diffraction spots
           do 161 nd=1,ndiff
                xc(1)=xold(1,nd)*amag
                xc(2)=xold(2,nd)*amag
                rc=fc*rdiff
                call fillarea(1)
                call fillcolor(0)
                call circle(0.,360.,rc,xc,icol)
                call fillarea(0)
 161       continue
      return
      end

      function jslct(hh)
      integer hh(*)
      real h(6)
c      common /ccase/ icase, ifix, hmin, hmax
      common /ccase/ icase, ifix, hmin, hmax, a1(3), a2(3), a3(3)
      common /kwave/ y(3,3,3),vc(3)
      data eps/0.00001/
      h(1)=hh(1)+y(1,1,1)*hh(4)+y(1,2,1)*hh(5)+y(1,3,1)*hh(6)
      h(2)=hh(2)+y(2,1,1)*hh(4)+y(2,2,1)*hh(5)+y(2,3,1)*hh(6)
      h(3)=hh(3)+y(3,1,1)*hh(4)+y(3,2,1)*hh(5)+y(3,3,1)*hh(6)
      if(icase.eq.1) then
            if(h(3).gt.hmin.and.h(3).lt.hmax) then
                jslct = 1
                return
            end if
      else if(icase.eq.2) then
            if(h(1).gt.hmin.and.h(1).lt.hmax) then
                jslct = 1
                return
            end if
      else if(icase.eq.3) then
            if(h(2).gt.hmin.and.h(2).lt.hmax) then
                jslct = 1
                return
            end if
      else if(icase.eq.4) then
            if(h(1)-h(2).gt.hmin.and.h(1)-h(2).lt.hmax) then
                jslct = 1
                return
            end if
      else if(icase.eq.5) then
            call yyy(h)
            eps=vinp(a3,h,3)
            if(eps.gt.hmin.and.eps.lt.hmax) then
                jslct = 1
                return
            end if
      else
            write(6,*) 'icase invalid'
            stop
      end if
      jslct = 0
      return
      end

      subroutine trcs(hh,xc)
      integer hh(*)
      real xc(*)
      real h(6)
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *        as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
c      common /ccase/ icase, ifix, hmin, hmax
      common /ccase/ icase, ifix, hmin, hmax, a1(3), a2(3), a3(3)
      common /kwave/ y(3,3,3),vc(3)

      h(1)=hh(1)+y(1,1,1)*hh(4)+y(1,2,1)*hh(5)+y(1,3,1)*hh(6)
      h(2)=hh(2)+y(2,1,1)*hh(4)+y(2,2,1)*hh(5)+y(2,3,1)*hh(6)
      h(3)=hh(3)+y(3,1,1)*hh(4)+y(3,2,1)*hh(5)+y(3,3,1)*hh(6)

      if(icase.eq.1) then
           xc(1)=h(1)*as+h(2)*bs*coscs
           xc(2)=h(2)*bs*sincs
      else if(icase.eq.2) then
           xc(1)=h(2)*bs+h(3)*cs*cosas
           xc(2)=h(3)*cs*sinas
      else if(icase.eq.3) then
           xc(1)=h(3)*cs+h(1)*as*cosbs
           xc(2)=h(1)*as*sinbs
      else if(icase.eq.4) then
           xc(1)=(h(1)*as+h(2)*bs)*sqrt(coscs+1)/sqrt(2.)
           xc(2)=h(3)*cs
      else if(icase.eq.5) then
           call yyy(h)
           xc(1)=vinp(a1,h,3)
           xc(2)=vinp(a2,h,3)
      else
           xc(1)=0
           xc(2)=0
      end if
      return
      end


      subroutine matinv(a,nm,n,b,m,determ,ipivot,index,pivot)
c
c    n*n matrix a is replaced by its inverse matrix a**-1
c    if m is nonzero vector b is replaced by a**-1*b
c
      dimension a(nm,nm),b(nm),ipivot(nm),index(nm,2),pivot(nm)
      equivalence (irow,jrow),(icolum,jcolum),(amax,t,swap)
      determ=1.0
      do 20 j=1,n
  20  ipivot(j)=0
      do 555 i=1,n
      amax=0.0
      do 105 j=1,n
      if(ipivot(j).eq.1) go to 105
      do 100 k=1,n
      if(ipivot(k)-1) 80,100,740
  80  if(abs(amax).ge.abs(a(j,k))) go to 100
      irow=j
      icolum=k
      amax=a(j,k)
 100  continue
 105  continue
      ipivot(icolum)=ipivot(icolum)+1
      if(irow.eq.icolum) go to 260
      determ=-determ
      do 200 l=1,n
      swap=a(irow,l)
      a(irow,l)=a(icolum,l)
 200  a(icolum,l)=swap
      if(m.eq.0) go to 260
      swap=b(irow)
      b(irow)=b(icolum)
      b(icolum)=swap
 260  index(i,1)=irow
      index(i,2)=icolum
      pivot(i)=a(icolum,icolum)
      determ=determ*pivot(i)
      a(icolum,icolum)=1.0
      do 350 l=1,n
 350  a(icolum,l)=a(icolum,l)/pivot(i)
      if(m.eq.0) go to 380
      b(icolum)=b(icolum)/pivot(i)
 380  do 550 l1=1,n
      if(l1.eq.icolum) go to 550
      t=a(l1,icolum)
      a(l1,icolum)=0.0
      do 450 l=1,n
 450  a(l1,l)=a(l1,l)-a(icolum,l)*t
      if(m.eq.0) go to 550
      b(l1)=b(l1)-b(icolum)*t
 550  continue
 555  continue
      do 710 i=1,n
      l=n+1-i
      if(index(l,1).eq.index(l,2)) go to 710
      jrow=index(l,1)
      jcolum=index(l,2)
      do 700  k=1,n
      swap=a(k,jrow)
      a(k,jrow)=a(k,jcolum)
 700  a(k,jcolum)=swap
 710  continue
 740  continue
      return
      end

      subroutine savexc(xc,xold,ndiff)
      real xc(*),xold(2,*)
      xold(1,ndiff)=xc(1)
      xold(2,ndiff)=xc(2)
      return
      end

      subroutine savehr(hr,hold,ndiff)
      integer hr(*),hold(6,*)
      do 100 i=1,6
 100  hold(i,ndiff)=hr(i)
      return
      end

      subroutine quit(str)
      character*(*) str
      write(6,*) str
      close(unit=3)
      close(unit=4)
      close(unit=91)
      close(unit=92)
      stop
      end

      subroutine ihkln(ih,hkln)
      integer ih(6),hkln,hm
      common /chkln/ iex(6),hm(6)
      ih(1)=hkln/iex(1)-hm(1)
      do 10 i=2,6
 10   ih(i)=mod(hkln,iex(i-1))/iex(i)-hm(i)
      return
      end


      subroutine norml(x,n)
      real x(*)
      abs=0.
      do i=1,n
         abs=abs+x(i)**2
      end do
      abs=sqrt(abs)
      do i=1,n
         x(i)=x(i)/abs
      end do
      return
      end
