c
c
c     refinement of modulated structures by the least squares method
c
c                           (REMOS 97.0)
c
c             copyright (c) oct. 1997  by Akiji Yamamoto
c
c                       input data instruction
c             
c             This program calculates refinable parameters and
c           constrained conditions for atoms at special positions
c
c             In the output list
c                 +1xc2 means amplitude of cosine wave for the x
c                 coordinate is refinabel.
c                 Similarly, y,z,p,b,b1,b2,b3,b4,b5 mean the y,z coordinates
c                 the isotropic and anisotropic temperature factors
c                   (b1=b11,b2=b22,b3=b33,b4=b23,b5=b31,b6=b12)
c                 For sine wave, s is used.
c                 
c                 The first digit (1 in this case) is a
c                 factor multiplied to the corresponding
c                 independent parameter
c                 3 is special and does not mean that the factor
c                 is 3 but sqrt(3)=1.73205...

c                 The last digit (2) stands for the modulation wave
c                 number in the 'kstar' line
c                 +1xc2=+3xs2 means that amplitudes for x and y coordinates
c                 are correlated and their ratio should be 1:sqrt(3).
c                 +1xc2=-2xs2 means that amplitudes for x and y coordinates
c                 are correlated and their ratio should be 1:-2.
c
c              Parameters to be fixed are not listed.
c
c
c     (1) title 
c
c     (2) a,b,c,cosa,coab,coac
c             a,b,c in angstrome
c     (3) na , nsub,  zmtrx
c         na: number of independent atoms in an asymmetric unit
c             of the aberage (fundamental) structure (1<=na<=100)
c         nsub: number of subsystems  
c               (for usual modulated structure nsub=1)
c               (for composite crystals, 2 <= nsub <=3)
c         zmtrx(1-6): permutation for the second to nsub-th subsystems
c         for example 1 3 4 2 5 6   2 3 5 1 4 6 means that 
c             a1*, a3*, a4* and a2*, a3*, a5* span the reciprocal lattices
c             for the second and third substructures
c             (for the first substructure 1 2 3 4 5 6 is assumed)
c
c     (4) 'basic structure'
c            no,symbol,sft,x1-x3,b,p,isub   (*)
c         no: atom number
c         symbol: symbol of the atomic site
c         stf: scattering-factor-table number
c         x1-x3: atomic coordinates
c         (for composite crystals, give coordinates for the substructure)
c         b: isotropic temperature factor
c         p: occupation probability
c         isub: subsystem  (1-3 for first to third subsystem)
c
c          =========== repeat (4) na times ===========
c
c     (5) ndm,nsymo,icent,ige,nctr,mmax
c         ndm: dimension of modulation     (0<=ndm<=3)
c               for average structure ndm=0
c         ***************
c         if negative value is given, printouts possible Fourier
c            terms for each atom in |ndm| case and then stops
c         ***************
c        
c         nsymo: number of symmetry operators in modulated structure
c                 (1<=nsymo<=48)
c         icent=0: noncentrosymmetric, =1: centrosymmetric
c           if centrosymmetric take the origin at center of symmetry
c           and give symmetry operators in the factor group, g/i
c           ige=0: give all symmetry opeators, ige=1: give generators,
c           then all elements are created in the program
c         nctr: number of centering translations  (0<=nctr<=48)
c               nctr=0 means primitive lattice
c         mmax: muximum satelite indeces (for each modulation wave)
c                (maximum miller indices for k1,k2,k3)
c          ********if ndm=0 skip (6)********
c     (6)  a*,b*,c* components of the wave vector
c           =========== repeat (6) ndm times ============
c         ====if(nctr=0) skip (7)
c     (7) 'centering translation'
c           in the format of I.T.
c           1/2,1/2,0,1/2  etc.
c           =========repeat (7) nctr times ========
c     (8) 'symmetry operator'
c           in the format of I.T.    (excluding 0,0,0,0,0,0)
c            for example x,y,z       etc. for ndm=0
c            for example x,y,z,t     etc. for ndm=1
c            for example x,y,z,t,u   etc. for ndm=2
c            for example x,y,z,t,u,v etc. for ndm=3
c         if ige=1 then give the order of the symmetry operator for ior
c           ======== repeat (8) nsymo times ==========
c         ****the first operator must be the identity operator****
c      (9) ix,,nds,nzc,nxmax
c         ix: no. of independent structural parameters
c         nds: no. of data sets ( no. of scale factprs )  (1<=nds<=20)
c         nzc: upper limit of the number of coefficients which relate
c          constrained atomic parameters and independent parameters
c         nxmax: upper limit of no. of variable parameter in each block
c     (10) itempf
c          itempf=0 isotropic temperature factor for all atoms
c          itempf=1 anisotropic temperatire factor Bij for all atoms
c          itempf=-1 some atoms anisotropic (Bij) and others isotropic
c          itempf=2 anisotropic temperatire factor Uij for all atoms
c          itempf=-2 some atoms anisotropic (Uij) and others isotropic
c               Beq=(4/3)(B11g11*+...+2B12g12*+...)
c               Ueq=(1/3)(U11g11g11*+..2U12g12g12*+...)
c               where gij and gij* are metric tensor
c ************************if itempf < 0 data ***************************
c *         specify anisotropic temperature factor                     *
c *    specify atoms with anisotropic temp.fact.                       *
c *    (11) (itfs(i),i=1,na)  (80i1)                                   *
c *    itfs(i)=0:temp.factor of i-th atom is isotropic,=1: anisotropic *
c **********************************************************************
c

      common /cdim/ idm,nde,ied
      common /cwflag/ iwflag(6)
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      common /cctwn/ dt(6,48)
      common /asft/ ne,ascf(35,20),df1(20),df2(20),iand,iel(3,100)
      integer dspmdl,tmpmdl,sctmdl
      common /cnsti/ ix,itvr,nds,na8,na9
      common /cnsto/ ix1,ix2,ix3,ix4,ix5,ix6,ix7,ix8,ix9,ix10,ix11,ix12
     *          ,ix13,ix14,ix15,ix16,ix17,ix18,ix19,ix20,ix21,ix22,
     *           ix23,ix24,ix25,ix26,ix27,ix28
      common /cnstt/ jx1,jx2,jx3
      common /cnst1/ kx1,kx2,kx3,kx4,kx5,kx6,kx7
      common /citfs/ itfs(100)
      real mu
      common /ctlint1/ nx,nzc,nl,isext,nxmax
      common /ctlint2/ nt
      common /ctlint3/ ipnf1,ipnf2,ipnf3
      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      integer mmax(3)
      parameter (memax=500000)
      dimension ax(memax),tt0(6)

      parameter (nmax=10000)
      character bx(nmax)

      parameter (lmax=10000)
      integer cx(lmax)

      integer defiwf(6)
      character*4 watom
      data watom/'atom'/,tt0/6*0.0/
      data defiwf/1,1,1,1,3,3/
      data zmtrx /1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6/

c    memory design  (the large array, ax, are devided into 25 parts
c    and each part is used as a different array in several subroutines)
c  ax(na2):mpl, ax(na3):x,
c  ax(na6):b,ax(na7):p, ax(ix1):x1
c  ax(ix2):x2, ax(ix3):x3, ax(ix4):x4, ax(ix5):x5, ax(ix6):x6
c  ax(ix7):ea, ax(ix8):xjl, ax(ix9):bjl, ax(ix10):fjl, ax(ix11):csb
c  ax(ix12):m, ax(ix13):l,
c  ax(ix17):cdr, ax(ix18):cdi,ax(ix19):xx,
c  ax(ix23):coeff, ax(ix24):csb1, ax(ix25):csb2
c  ax(ix26):hl,ax(ix27):dl

c  bx(1):iw, bx(jx1):mx, bx(jx2):ifrt1       ! character

c  cx(1):isft,cx(kx1):isk,cx(kx2):irs,     ! integer 
c  cx(kx3):isub,cx(kx4):ips,cx(kx5):ja,
c  cx(kx6):ifrt2

      call setinp(5)
      call setout(6)
      call setskp(0)
c calculation of sin cosin table
c     call sctabl
c calculation of exp table
c     call sexpt
c (0) read output control flags
c      read(5,'(6i1)') (iwflag(i),i=1,6)
      do 10 i=1,6
 10      iwflag(i)=defiwf(i)
c     write(6,'(1h ,a,6i3)') 'output control flags  ', iwflag

c (1),(2) read title and lattice constants
      call rdlc
c (3) read number of atoms in an asymmetric unit
c      read(5,100) na
      call freadi('na=',1,na,1)
      call freadi('nsub=',1,nsub,0)
      idm=abs(nde)+3
      if(nsub.le.0.or.nsub.gt.3) call quit('*** nsub invalid***')
      if(nsub.ge.2) then
          call freadi('zmtrx=',6,zmtrx(1,2),0)
      end if
      if(nsub.eq.3) then
          call freadi('zmtrx=',6,zmtrx(1,3),0)
      end if
      if(iwflag(3).eq.1) then
           write(6,*) 'na=',na,' nsub=',nsub,
     *        (' zmtrx=',(zmtrx(i,n),i=1,6),n=2,nsub)
      end if

cc  ax(na1):isft,ax(na4):isk,ax(na5):irs,    ! integer 
cc  ax(na8):isub,ax(ix14):ips,ax(ix15):ja,
cc  ax(ix22):ifrt2

      na1=1
      na2=na1
      na3=na+na2
      na4=6*na+na3
      na5=na4
      na6=na5
      na7=na+na6
      na8=na+na7
      na9=na8

      kx0=1
      kx1=kx0+na
      kx2=kx1+49*na
      kx3=kx2+49*na
      kx4=kx3+na

c (4) read atomic positions

      call rdap(na,cx(1),ax(na3),ax(na6),ax(na7),cx(kx3))

c (5)-(10) read symmetry operators
      call rdsymo(nw,mmax)
      call getgij(zmtrx,nsub)
      call btobij(na,ax(na6), cx(kx3))
      call setiski

      call sitesy(na,ax(na2),ax(na3),cx(kx1),cx(kx2),cx(kx3))
      if(nde.eq.0) then
         mmax(1)=0
         mmax(2)=0
         mmax(3)=0
      end if

       ix=1000
       nxmax=5000

       ix=na*nw*10   ! x,y,z,p,b1,b2,b3,b4,b5,b6
       nzc=ix

c       call freadi('nxmax=',1,nxmax,0)
c       write(6,*) 'ix=',ix,' nds=',nds,' nzc=',nzc,' nxmax=',nxmax
c      if(nxmax.eq.0) nxmax=250
c (12) read control variables
c      read(5,102) dspmdl,tmpmdl,sctmdl,itempf,itwin,iphfl,isext
      call freadi('itempf=',1,itempf,1)
      call getnde(nde)

      write(6,*) 'itempf=',itempf

c (13)
      if(itempf.lt.0) read(5,103) (itfs(i),i=1,na)
      if(itempf.lt.0) write(6,201) (watom,i,itfs(i),i=1,na)

c     printout possible fourier terms and then stop
      call splpos(na,cx(kx1),cx(kx2),cx(kx3),mmax)

      stop
 100  format(14i3)
 102  format(7i3)
 103  format(80i1)
 200  format(1h ,'lack of dimension',10x,'ax =',i6)
 201  format(1h ,'selection of temperature factors, 0: isotropic,',
     *      ' 1:anisotropic'/(1h ,10(a4,i2,':',i1,2x)))
 206  format(1h ,'dspmdl=',i3,3x,'tmpmdl=',i3,3x,'sctmdl=',i3,3x,
     *      'itempf=',i3,3x,'itwin=',i3,3x,'iphfl=',i3,3x,'isext=',i3)
      end

      subroutine rdap(na,isft,x,b,p,isub)
      common /cwflag/ iwflag(6)
      character*80 dummy
      character*6 atom
      common /cltc/ atom(100)
      dimension isft(*),x(6,*),b(*),p(*),isub(*)
      call getinp(inpu)
      call getout(iout)
      call freadc('basic structure',0,dummy,1)
      if(iwflag(3).eq.1) then
          write(6,301)
      end if
      do i=1,na
         read(inpu,*) ik,atom(i),isft(i),(x(j,i),j=1,3),b(i),
     *        p(i),isub(i)
         do j=4,6
            x(j,i)=0
         end do
         if(iwflag(3).eq.1) then
            write(iout,302) ik,atom(i),isft(i),(x(j,i),j=1,3),
     *           b(i),p(i),isub(i)
         end if
      end do
      if(iwflag(3).eq.1) then
           write(iout,300)
      end if
 10   continue
 102  format(i2,a4,a2,i2,7f9.6)
 300  format(1h )
 301  format(1h ,'  No',2x,'atom',2x,'s.f.',4x
     * ,'x',8x,'y',8x,'z',8x,'b',8x,'p',6x,'isub')
 302  format(1h ,i4,1x,a,1x,i2,1x,3f9.5,1x,2f9.3,3x,i1)
      return
      end

      subroutine rdfour(nw,nt,ifrt1,ifrt2,coeff)
      common /cwflag/ iwflag(6)
      character ifrt1(nw,2,nt)
      integer ifrt2(nw,3,nt)
      real coeff(nw,nt)
      character*256 str
      character*80 token(100)
      character*80 dummy

      call getnde(nde)
      call getinp(inpu)
      call getout(iout)

      if(nde.eq.0) then
         ifrt1(1,1,1)='+'
         ifrt1(1,2,1)='c'
         ifrt2(1,1,1)=0
         ifrt2(1,2,1)=0
         ifrt2(1,3,1)=0
         coeff(1,1)=1.0
      else
         call freadc('fourier term',0,dummy,1)
         if(iwflag(3).eq.1) then
            write(iout,*) 'fourier term'
            write(6,*) 'nw=',nw/2
         end if
         i=1
         k=1
         do nn=1,nw/2
            read(inpu,*) i1,i2,i3
            write(iout,'(3i3)') i1,i2,i3
            if(i1.eq.0.and.i2.eq.0.and.i3.eq.0) then
               if(nn.ne.1) then
                  call quit('0 0 0 should be the first fourier term')
               end if
            end if
            n=(nn-1)*2+1
            ifrt1(n,1,i)='+'
            ifrt1(n,2,i)='c'
            ifrt2(n,1,i)=i1
            ifrt2(n,2,i)=i2
            ifrt2(n,3,i)=i3
            coeff(n,i)=1.0
            n=(nn-1)*2+2
            ifrt1(n,1,i)='+'
            ifrt1(n,2,i)='s'
            ifrt2(n,1,i)=i1
            ifrt2(n,2,i)=i2
            ifrt2(n,3,i)=i3
            coeff(n,i)=1.0
         end do
      end if
      return
      end

      subroutine rdfour0(nw,nt,ifrt1,ifrt2,coeff)
      common /cwflag/ iwflag(6)
      character ifrt1(nw,2,nt)
      integer ifrt2(nw,3,nt)
      real coeff(nw,nt)
      character*256 str
      character*80 token(100)
      character*80 dummy

      call getnde(nde)
      call getinp(inpu)
      call getout(iout)

      if(nde.eq.0) then
         ifrt1(1,1,1)='+'
         ifrt1(1,2,1)='c'
         ifrt2(1,1,1)=0
         ifrt2(1,2,1)=0
         ifrt2(1,3,1)=0
         coeff(1,1)=1.0
      else
         call freadc('fourier term',0,dummy,1)
         if(iwflag(3).eq.1) then
            write(iout,*) 'fourier term'
            write(6,*) 'nw=',nw
         end if
         i=1
         k=1

 10      continue
         read(inpu,'(a)') str
         call gettoken(str,token,ntk)
c        write(6,*) (token(j),j=1,ntk)
         do jd=1,ntk    !   +c 1 0 0& etc
            j=mod(jd-1,4)+1
            is=istrln(token(jd))
            if(j.eq.1) then
               ifrt1(i,1,k)=token(jd)(1:1)    ! + or -
               ifrt1(i,2,k)=token(jd)(2:2)    ! c or s
            else if(j.eq.2.or.j.eq.3) then
               read(token(jd)(1:is),'(i3)') ifrt2(i,j-1,k)
            else if(j.eq.4) then
               if(token(jd)(is:is).eq.'&') then
                  read(token(jd)(1:is-1),'(i3)') ifrt2(i,j-1,k)
                  if(ifrt1(i,1,k).eq.'+') then
                     coeff(i,k)=1.0
                  else
                     coeff(i,k)=-1.0
                  end if
                  k=k+1
               else
                  read(token(jd)(1:is),'(i3)') ifrt2(i,j-1,k)
                  if(ifrt1(i,1,k).eq.'+') then
                     coeff(i,k)=1.0
                  else
                     coeff(i,k)=-1.0
                  end if
                  write(6,*) (ifrt1(i,1,n),ifrt1(i,2,n),
     *                 ifrt2(i,1,n),ifrt2(i,2,n),ifrt2(i,3,n),n=1,k)
                  i=i+1                  
                  k=1
               end if
            end if
         end do
         if(i.le.nw) go to 10
      end if
      return
      end

      subroutine rdintl(x,mx,ja,ea,x1,isub,na,nw,nc,nu,ips)
      character mx(na,nc,*)
      real x(na,nc,*),ea(*),x1(*)
      integer ja(4,*),isub(*)
      real twdr(100)
      character mtwdr(100)
      integer ips(*)
      common /cwflag/ iwflag(6)
      common /const/ pi2,i7,i8,i9,i10,i11,i12,i13,i14
      common /cnsti/ ix,itvr,nds,na8,na9
      character*6 atom
      common /cltc/ atom(100)
      character*6 atom1(100)
      integer dspmdl,tmpmdl,sctmdl
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      common /citfs/ itfs(100)
      common /ceat/ eat(6,3)
      common /ctlint1/ nx,nzc,nl,isext,nxmax
      common /ctlint2/ nt
      character*4 cmp1,cmp2,cmp3,cmp4,aa,no
      character*80 dummy,str
      character*20 token(100)
      character*80 xyzpb(1000)
      character*256 twnvr,strscl
      real tw(100)
      integer ipmsg(0:48)
      character mt(100)
      integer ltk(100)

      integer ixx(10,10,20)

c      dimension comp1(3),comp2(7),comp3(12),comp4(1)
      data no/'#'/
c      data comp1/'u','v','w'/
c      data comp2/'B','B44d','B55d','B66d','B56d','B64d','B45d'/
c      data comp3/'B1','B2','B3','B4','B5',
c     *      'B6','B44d','B55d','B66d','B56d','B64d','B45d'/
c      data comp4/'p'/
c*******************************************************************
c                      read initial parameters
c*******************************************************************
      call getinp(inpu)
      call getout(iout)
c      call freadc('initial parameter',0,dummy,1)
      read(inpu,'(a)') str
c*******************************************************************
c             read independent initial amplitudes
c*******************************************************************

c     read nonconstrained parameters
c        deviation for
c        x,y,z,p,B or                 (itempf=0 or itempf=-1 and itfs=0)
c        x,y,z,p,B1,B2,B3,B4,B5,B6 or (itempf=1 or itempf=-1 and itfs=1)
c        x,y,z,p,U or                 (itempf=2 and itfs=0)
c        x,y,z,p,U1,U2,U3,U4,U5,U6    (itempf=2 or itempf=-2 and itfs=1)

c      write(6,*) '*****na,nw,nc=',na,nw,nc,'*****'

      do i=1,na
         do kd=1,nw/2
            ik=kd+(i-1)*nw/2
            read(inpu,'(a)') xyzpb(ik)
            is=istrln(xyzpb(ik))
            write(6,*) xyzpb(ik)(1:is)
         end do
      end do

      do i=1,na
         do j=1,10
            do k=1,nw
               ixx(i,j,k)=0
            end do
         end do
      end do

      nu=0   ! number of independent parameters
      ii=0   ! number of parameters

      do i=1,na
c        read independent parameters
         do kd=1,nw/2
            ik=kd+(i-1)*nw/2
            call gettoken(xyzpb(ik),token,ntk)
            if(kd.eq.1) then
               ntk1=ntk
            else if(kd.gt.1) then
               if(itempf.eq.0) then
                  ntk1=7
               else if(itempf.gt.0) then
                  ntk1=12
               else if(itempf.lt.0) then
                  if(itfs(i).eq.0) then
                     ntk1=7
                  else if(itfs(i).eq.1) then
                     ntk1=12
                  end if
               end if
            end if
c           write(6,*) xyzpb(ik)
c           token : Al etc, number or =+1zc3+0.235 etc.
            do jd=1,ntk
               if(jd.le.ntk1) then
                  j=jd-2       ! skip atom and ft
                  k=(kd-1)*2+1 ! cosine term
               else if(jd.gt.ntk1) then
                  j=jd-ntk1
                  k=(kd-1)*2+2 ! sine term
               end if
               is=istrln(token(jd))
               call getpmsg(token(jd),is,ipmsg,npmsg) ! position of + or -
               ipm=ipmsg(npmsg) ! last + or - position
c               write(6,'(a,5i3)') 'i,j,k,kd,isnumber=',i,j,k,kd,
c     *              isnumber(token(jd)(ipm+1:is))
               if(jd.eq.1) then
                  atom1(i)=token(jd)(1:is)   ! atom
               else if(jd.eq.2) then
                  !  skip number             ! ft
               else if(jd.ge.3.and.isnumber(token(jd)(ipm+1:is)).eq.1)
     *                 then   ! independent parameter
                  nu=nu+1
                  ii=ii+1
                  ja(1,ii)=i   ! atom number
                  ja(2,ii)=j   ! x,y,z,p,b etc
                  ja(3,ii)=k   ! cos or sin term
                  ea(ii)=1.0
                  ja(4,nu)=1
                  if(token(jd)(is:is).ne.'*') then
                     read(token(jd)(1:is),'(f10.5)') x(i,j,k)
                     mx(i,j,k)=' '
                     ixx(i,j,k)=1
                     ips(nu)=1
                  else
                     read(token(jd)(1:is-1),'(f10.5)') x(i,j,k)
                     mx(i,j,k)='*'
                     ixx(i,j,k)=1
                     ips(nu)=0
                  end if
                  x1(nu)=x(i,j,k)  ! independent parameter
                  call cnstprm(nw,xyzpb,ja,ea,i,j,k,ii,nu,x1(nu),
     *                 x,mx,na,nc,ixx)
               end if
            end do
         end do
      end do

c      do i=1,na
c         do k=1,nw
c            write(6,'(60i1)') (ixx(i,j,k),j=1,10)
c         end do
c      end do

      ixpb=nu
      iixpb=ii

      iw=0
      call prmtbl(x1,df,isub,ips,na,nw,ja,ea,mx,nc,iw)
c
      write(6,*) 'parm. atom comp. wave coeff.'
      in=0
      do n=1,ixpb
         nz=ja(4,n)
         write(6,'(5(a,i4,3i3,f9.5))') (' #',n,ja(1,in+j),ja(2,in+j),
     *        ja(3,in+j),ea(in+j),j=1,nz)
         in=in+nz
      end do

      call setix(nu)

      if(iwflag(3).eq.1) then
           write(iout,*) 'nu=',nu
      end if

      itvr=0
      if(itwin.ne.0) then
         call freadc('twin parameter',0,dummy,1)
c        read independent parameters
         read(inpu,'(a)') twnvr
         call gettoken(twnvr,token,ntk)
c        token : number or =+1-1z3 etc.
c        write(6,*) (token(jj)(1:istrln(token(jj))),jj=1,ntk)

         if(ntk.ne.itwin) then
            call quit('twin parameter invalid')
         end if

         do jd=1,ntk   ! ntk=itwin
            is=istrln(token(jd))
            call getpmsg(token(jd),is,ipmsg,npmsg) ! position of + or -
            ipm=ipmsg(npmsg) ! last + or - position
            write(6,*) 'i,j,k,isnumber=',i,j,k,
     *           isnumber(token(jd)(ipm+1:is))
            if(isnumber(token(jd)(ipm+1:is)).eq.1)
     *                 then   ! independent parameter
               ii=ii+1
               ja(1,ii)=jd
               ja(2,ii)=1
               ja(3,ii)=1
               ea(ii)=1.0
               if(token(jd)(is:is).ne.'*') then   ! independent twin parameter
                  read(token(jd)(1:is),'(f10.5)') tw(jd)
                  mt(jd)=' '
                  nu=nu+1
                  ja(4,nu)=1
                  ips(nu)=1
               else  ! independent twin parameter but is fixed
                  read(token(jd)(1:is-1),'(f10.5)') tw(jd)
                  mt(jd)='*'
                  nu=nu+1
                  ja(4,nu)=1
                  ips(nu)=0
               end if
               x1(nu)=tw(jd)   ! independent parameter
               call cnsttwn(twnvr,ja,ea,jd,ii,nu,x1(nu),x,mx,na,nc)
            end if
         end do
         itvr=nu-ixpb
         call setvolr(itwin,ja,ea,x1,iixpb,ixpb,itvr)
      end if

      i7=ixpb+1
      i8=ixpb+itvr

      i9=i8+1   ! dummy
      i10=i8    ! dummy

      read(inpu,'(a)') strscl
      call gettoken(strscl,token,ntk)
      do jd=1,ntk      ! ntk=nds+1  first one will be scl
         is=istrln(token(jd))
         if(jd.eq.1) then
            if(token(1)(1:is).ne.'scl') then
               call quit('scl not found')
            end if
         else
            if(token(jd)(is:is).ne.'*') then
               nu=nu+1
               ii=ii+1
               read(token(jd)(1:is),'(f10.5)') x1(nu)
               ips(nu)=1
            else
               nu=nu+1
               ii=ii+1
               read(token(jd)(1:is-1),'(f10.5)') x1(nu)
               ips(nu)=0
            end if
         end if
      end do

      i11=i10+1
      i12=i10+nds
   
      i13=i12+1
      i14=i12
      if(isext.ne.0) then
         ii=ii+2
         nu=nu+2
         i13=i12+1
         i14=i12+2
      end if

      call getavds(avds)
      call getavma(avma)
      x1(i13)=avds
      x1(i14)=avma

      call setnu(nu)

      return
 100  format(a,10(f9.4,a1))
 101  format(8(f10.5))
 112  format(3i3)
 212  format(10(f10.5))
      end

      function isnumber(token)
c     return 1 if token is an integer or a real number
c          (3 or 0.125 etc.) else 0  (1.0e-1 etc. is not allowed)
      character*(*) token
      character ia(0:10)
      data ia /'0','1','2','3','4','5','6','7','8','9','.'/

      is=istrln(token)
      if(token(is:is).eq.'*') then   ! strip the last '*' if any
         is=is-1
      end if

      do i=1,is
         do j=0,10
            if(token(i:i).eq.ia(j)) then
               go to 100
            end if
         end do
         isnumber=0
         return
 100     continue
      end do
      isnumber=1
      return
      end

      subroutine getvolr(itwin,volrd)
      real volrd(*)
      common /cvolr/ volr(48)
      do i=1,itwin
         volrd(i)=volr(i)
      end do
      return
      end

      subroutine setvolr(itwin,ja,ea,x,iy0,ix0,itvr)
      integer ja(4,*)
      real ea(*),x(*)
      common /cvolr/ volr(48)

      in=iy0
      do i=1,itvr
         nz=ja(4,i+ix0)
         do n=1,nz
            in=in+1
            itw=ja(1,in)
c           write(6,*) 'i,n,itw',i,n,itw
            volr(itw)=volr(itw)+x(i+ix0)*ea(in)
         end do
      end do

      write(6,'('' twin domain volume ratio: '',8f12.6)')
     *     (volr(itw),itw=1,itwin)
      write(6,*) ' '
      return
      end

      subroutine setvolr0(itwin)
      common /cvolr/ volr(48)
      volr(1)=1.0
      return
      end

      subroutine cnsttwn(twnvr,ja,ea,i,ii,nu,x1,x,mx,na,nc)
      character*(*) twnvr
      character*20 token(100)
      integer ja(4,*)
      real ea(*)
      real x1
      real x(na,nc,*)
      character mx(na,nc,*)
      integer ipmsg(0:48)
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      common /ctwin/ ita(48),fa(48)  ! constant term for twin

c     write(6,*) 'enter cnsttprm'
      call gettoken(twnvr,token,ntk)
c     token : number or =-1t3+1.0* etc.
      do jd=1,ntk       ! ntk=itwin
         if(ntk.ne.itwin) then
            call quit('twin parameter invalid')
         end if
         is=istrln(token(jd))    ! length of the string
         call getpmsg(token(jd),is,ipmsg,npmsg) ! position of + or -
         write(6,*) 'npmsg=',npmsg,(ipmsg(n),n=1,npmsg)
         if(token(jd)(1:1).eq.'=') then ! =-1t2-1t3+1.0* etc
            do n=1,npmsg
               ipm=ipmsg(n)
               if(token(jd)(ipm+2:ipm+2).eq.'t') then
                  ja1=jd  ! twin domain number
                  ja2=1
                  read(token(jd)(ipm+3:ipm+3),'(i2)') ja3
                  read(token(jd)(ipm+1:ipm+1),'(i2)') iea
                  if(iea.eq.1) then
                     ea(jd)=1
                  else if(iea.eq.-1) then
                     ea(jd)=-1
                  else
                     write(6,*) token(jd)(1:is)
                     call quit('invalid twin parameter')
                  end if
               else if(ipm.ne.ipmsg(npmsg)) then
                  write(6,*) token(jd)(1:is)
                  call quit('invalid twin parameter')
               end if                  
               if(ja3.eq.i) then
                  ii=ii+1
                  ja(1,ii)=ja1
                  ja(2,ii)=1
                  ja(3,ii)=1
                  read(token(jd)(2:3),'(i2)') iea
c                 write(6,*) token(jd)(2:3),iea
                  if(iea.eq.1) then
                     ea(ii)=1
                  else if(iea.eq.-1) then
                     ea(ii)=-1
                  else
                     write(6,*) token(jd)(1:is)
                     call quit('invalid twin parameter')
                  end if
                  ja(4,nu)=ja(4,nu)+1
c                 x(i,1,1)=x(i,1,1)+x1*ea(ii)
                  x(i,1,1)=x1*ea(ii)
                  mx(i,1,1)='&'
               end if
            end do
         end if
      end do
      return
      end

      subroutine getpmsg(token,is,ipmsg,npmsg) ! position of + or -
      character*(*) token
      integer ipmsg(0:*)
      npmsg=0
      ipmsg(0)=0
      do i=1,is
         if(token(i:i).eq.'+'.or.token(i:i).eq.'-') then
            npmsg=npmsg+1
            ipmsg(npmsg)=i
         end if
      end do
      return
      end

      subroutine cnstprm(nw,xyzpb,ja,ea,i,j,k,ii,nu,x1,x,mx,na,nc,ixx)
      integer ixx(10,10,*)
      character*(*) xyzpb(*)
      character*20 token(100)
      integer ja(4,*)
      real ea(*)
      real x1
      real x(na,nc,*)
      character mx(na,nc,*)
      integer ipmsg(0:48)
      character cs
      integer dspmdl,tmpmdl,sctmdl
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      common /citfs/ itfs(100)

c      write(6,*) 'enter cnstprm'
c      write(6,'(a,5i3)') 'i,j,k,ii,nu=',i,j,k,ii,nu
      ik0=(i-1)*nw/2

      do kd=1,nw/2
         ik=kd+ik0
         call gettoken(xyzpb(ik),token,ntk)
         if(kd.eq.1) then
            ntk1=ntk
         else if(kd.gt.1) then
            if(itempf.eq.0) then
               ntk1=7
            else if(itempf.gt.0) then
               ntk1=12
            else if(itempf.lt.0) then
               if(itfs(i).eq.0) then
                  ntk1=7
               else if(itfs(i).eq.1) then
                  ntk1=12
               end if
            end if
         end if
c        token : Al etc, number or =+1zc3-1ys2+1.0* etc.
         do jd=1,ntk
            is=istrln(token(jd))
            call getpmsg(token(jd),is,ipmsg,npmsg) ! position of + or -
            if(jd.le.ntk1) then
               j1=jd-2            ! x,y,z,p,b etc.
               k1=(kd-1)*2+1      ! cosine term
            else
               j1=jd-ntk1         ! x,y,z,p,b etc.
               k1=(kd-1)*2+2      ! sine term
            end if
            if(j1.eq.j.and.k1.eq.k) go to 110
            if(jd.ge.3.and.token(jd)(1:1).eq.'=') then ! =+1zc3-1y2+1.0* etc
c               write(6,*) token(jd)(1:is)
               do n=1,npmsg
                  ipm=ipmsg(n)
                  if(n.eq.npmsg) then
                     jpm=is+1
                  else
                     jpm=ipmsg(n+1)
                  end if
c                  ja2=0
c                  ja3=0
                  if(token(jd)(ipm+2:ipm+2).eq.'x') then
                     ja2=1
                     cs=token(jd)(ipm+3:ipm+3)
                     read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                  else if(token(jd)(ipm+2:ipm+2).eq.'y') then
                     ja2=2
                     cs=token(jd)(ipm+3:ipm+3)
                     read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                  else if(token(jd)(ipm+2:ipm+2).eq.'z') then
                     ja2=3
                     cs=token(jd)(ipm+3:ipm+3)
                     read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                  else if(token(jd)(ipm+2:ipm+2).eq.'p') then
                     ja2=4
                     cs=token(jd)(ipm+3:ipm+3)
                     read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                  else if(itempf.eq.0) then
                     if(token(jd)(ipm+2:ipm+2).eq.'b') then
                        ja2=5
                        cs=token(jd)(ipm+3:ipm+3)
                        read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                     else
                        write(6,*) token(jd)(1:is)
                        call quit('invalid constrained parameter')
                     end if
                  else if(itempf.eq.1.or.(itempf.eq.-1.and.
     *                    itfs(i).eq.1)) then
                     if(token(jd)(ipm+2:ipm+3).eq.'b1') then
                        ja2=5
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'b2') then
                        ja2=6
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'b3') then
                        ja2=7
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'b4') then
                        ja2=8
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'b5') then
                        ja2=9
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'b6') then
                        ja2=10
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else
                        write(6,*) token(jd)(1:is)
                        call quit('invalid constrained parameter')
                     end if
                  else if(itempf.eq.-1.and.itfs(i).eq.0) then
                     if(token(jd)(ipm+2:ipm+2).eq.'b') then
                        ja2=5
                        cs=token(jd)(ipm+3:ipm+3)
                        read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                     else
                        write(6,*) token(jd)(1:is)
                        call quit('invalid constrained parameter')
                     end if
                  else if(itempf.eq.2.or.(itempf.eq.-2.and.
     *                    itfs(i).eq.1)) then
                     if(token(jd)(ipm+2:ipm+3).eq.'u1') then
                        ja2=5
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'u2') then
                        ja2=6
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'u3') then
                        ja2=7
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'u4') then
                        ja2=8
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'u5') then
                        ja2=9
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else if(token(jd)(ipm+2:ipm+3).eq.'u6') then
                        ja2=10
                        cs=token(jd)(ipm+4:ipm+4)
                        read(token(jd)(ipm+5:jpm-1),'(i3)') ja3
                     else
                        write(6,*) token(jd)(1:is)
                        call quit('invalid constrained parameter')
                     end if
                  else if(itempf.eq.-2.and.itfs(i).eq.0) then
                     if(token(jd)(ipm+2:ipm+2).eq.'u') then
                        ja2=5
                        cs=token(jd)(ipm+3:ipm+3)
                        read(token(jd)(ipm+4:jpm-1),'(i3)') ja3
                     else
                        write(6,*) token(jd)(1:is)
                        call quit('invalid constrained parameter')
                     end if
                  end if
                  if(cs.eq.'c') then
                     ja3d=(ja3-1)*2+1
                  else if(cs.eq.'s') then
                     ja3d=(ja3-1)*2+2
                  end if
                  if(ja2.eq.j.and.ja3d.eq.k) then
c                     write(6,*) 'j1,k1,ja2,ja3d=',j1,k1,ja2,ja3d
                     ii=ii+1
                     ja(1,ii)=i
                     ja(2,ii)=j1
                     ja(3,ii)=k1
                     read(token(jd)(2:3),'(i2)') iea
c                    write(6,*) token(jd)(2:3),iea
                     if(iea.eq.1) then
                        ea(ii)=1
                     else if(iea.eq.-1) then
                        ea(ii)=-1
                     else if(iea.eq.2) then
                        ea(ii)=2
                     else if(iea.eq.-2) then
                        ea(ii)=-2
                     else if(iea.eq.3) then
                        ea(ii)=sqrt(3.)
                     else if(iea.eq.-3) then
                        ea(ii)=-sqrt(3.)
                     else
                        call quit('invalid constrained parameter')
                     end if
                     ja(4,nu)=ja(4,nu)+1
                     x(i,j1,k1)=x(i,j1,k1)+x1*ea(ii)
                     mx(i,j1,k1)='&'
                     ixx(i,j1,k1)=1
                  end if
               end do
            end if
 110        continue
         end do
      end do
      return
      end

      subroutine gettoken(str,token,ntk)
      character*(*) str,token(*)
      integer j1(100),j2(100)
      lstr=len(str)
      ntk=0
      i1=1
 100  continue
      do i=i1,lstr
         if(str(i:i).ne.' ') then
            ntk=ntk+1
            ! find first character of the ntk-th token
            j1(ntk)=i
            do j=i+1,lstr
               if(str(j:j).eq.' ') then
                   ! end of the ntk-th token
                  j2(ntk)=j-1
                  i1=j
                  go to 100
               end if
            end do
         end if
      end do
      do n=1,ntk
         token(n)=str(j1(n):j2(n))
      end do
      return
      end

      subroutine setix(ixd)
      common /csetix/ ix
      ix=ixd
      return
      end

      subroutine getix(ixd)
      common /csetix/ ix
      ixd=ix
      return
      end

      subroutine setnu(nud)
      common /csetnu/ nu
      nu=nud
      return
      end

      subroutine getnu(nud)
      common /csetnu/ nu
      nud=nu
      return
      end

      subroutine rdlc
      common /cwflag/ iwflag(6)
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *        as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      character*80 mater
      common /ctitle/ mater
      real ui(6,3),u1(3,3),u2(3,3)

      call getinp(inpu)
      call getout(iout)
c      read(inpu,99) mater
      call freadc('title=',1,mater,1)
c      read(inpu,100) a,b,c,cosa,cosb,cosc
      call freadr('a=',1,a,1)
      call freadr('b=',1,b,0)
      call freadr('c=',1,c,0)
      call freadr('cosa=',1,cosa,0)
      call freadr('cosb=',1,cosb,0)
      call freadr('cosc=',1,cosc,0)
      if(iwflag(2).eq.1) then
           write(iout,'(a)') mater
      end if
      if(iwflag(3).eq.1) then
           write(iout,300) a,b,c,cosa,cosb,cosc
      end if
      sina=sqrt(1.0-cosa**2)
      sinb=sqrt(1.0-cosb**2)
      sinc=sqrt(1.0-cosc**2)
      call calltcs(a,b,c,cosa,cosb,cosc,as,bs,cs,cosas,cosbs,coscs)
      icase=1
      call calu1(icase,as,bs,cs,cosas,cosbs,coscs,u1)
      do i=1,3
         do j=1,3
            u2(i,j)=0
         end do
      end do
      call setu1(u1)
      call setu2(u2)
c      call setui(ui)
      return
 100  format(6f12.6)
 300  format(1h ,'a=',f9.4,1x,'b=',f9.4,1x,'c=',f9.4,1x,
     *    'cosa=',f9.5,1x,'cosb=',f9.5,1x,'cosc=',f9.5)
      end

      subroutine trnsp(am)
      real am(3,3),bm(3,3)
      do 100 i=1,3
      do 100 j=1,3
          bm(j,i)=am(i,j)
 100  continue
      do 200 i=1,3
      do 200 j=1,3
          am(i,j)=bm(i,j)
 200  continue
      return
      end

      subroutine geticnt(icnt)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      icnt=icent
      return
      end

      subroutine getnde(nd)
      common /cdim/ idm,nde,ied
      nd=nde
      return
      end

      subroutine setidm(idmd)
      common /cdim/ idm,nde,ied
      idm=idmd
      return
      end

      subroutine getidm(idmd)
      common /cdim/ idm,nde,ied
      idmd=idm
      return
      end

      function icomp(h,hr)
      integer h(*),hr(*)
      common /cdim/ idm,nde,ied
      do i=1,idm
         if(h(i)-hr(i).ne.0) then
            icomp = 0
            return
         end if
      end do
      icomp = 1
      return
      end

      function icompn(h,hr,n)
      integer h(*),hr(*)
      do i=1,n
         if(h(i)-hr(i).ne.0) then
            icompn = 0
            return
         end if
      end do
      icompn = 1
      return
      end

      function icomprn(h,hr,n)
      real h(*),hr(*)
      data eps /0.0001/
      do i=1,n
         if(abs(h(i)-hr(i)).gt.eps) then
            icomprn = 0
            return
         end if
      end do
      icomprn = 1
      return
      end

      function icompcn(h,hr,n)
      complex h(*),hr(*)
      data eps /0.0001/
      do i=1,n
         if(abs(h(i)-hr(i)).gt.eps) then
            icompcn = 0
            return
         end if
      end do
      icompcn = 1
      return
      end

      function iextnct(h)
      integer h(*)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      common /cdim/ idm,nde,ied
      common /cctr/ ctr(6,0:48),nctr
      complex pi2
      integer hr(6)
      complex ff
      data pi2/(0.,6.28318)/
      data eps /0.01/
c     extinction due to centering translations
      if(nctr.gt. 0) then
         do i=1,nctr
            ff=cexp(pi2*vinpir(h,ctr(1,i),idm))
            if(abs(ff-1.0).gt.eps) then
               iextnct = 1
               return
            end if
         end do
      end if
c     extinction due to screw axes or glide planes
      do i=2,nsymo
        do j=0,icent
            call gethr(h,hr,i,j,1)
            if(icomp(h,hr).eq.1) then
               ff=cexp(pi2*vinpir(h,tt(1,i,1),idm))
               if(abs(ff-1.0).gt.eps) then
                  iextnct = 1
                  return
               end if
            end if
         end do
      end do

      iextnct = 0
      return
      end

      function vinpir(h,tt,idm)
      integer h(*)
      real tt(*)
      vinpir=0
      do i=1,idm
         vinpir=vinpir+h(i)*tt(i)
      end do
      return
      end

      subroutine getnsym(nsymod)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      nsymod=nsymo
      return
      end

      subroutine rdsymo(nw,mmax)
      integer mmax(*)
      common /cwflag/ iwflag(6)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      common /cctr/ ctr(6,0:48),nctr
      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      common /bgn/ ilm(3),icorc(3),ilc(3)
      common /kwave/ y(3,3,3),vc(3)
      common /cdim/ idm,nde,ied
      common /ctlint2/ nt
      dimension imps(48),fmt(2)
      integer rg(6,6,5)
      real tg(6,5)
      character*4 fmt
      character*80 dummy
      character*30 coord
      character*80 str
      data fmt/'in','  '/
      call getinp(inpu)
      call getout(iout)
c     initialozation
      do i=1,3
         ilm(i)=1
         ilc(i)=1
         icorc(i)=0
         do j=1,3
            y(j,i,1)=0.0
         end do
      end do
c      read(inpu,100) nde,ied,nsymo,icent,ige,nctr
      call freadi('ndm=',1,nde,1)
c      call freadi('ied=',1,ied,0)
      ied=0
      call freadi('nsymo=',1,nsymo,0)
      call freadi('icent=',1,icent,0)
      call freadi('ige=',1,ige,0)
      call freadi('nctr=',1,nctr,0)
      if(nde.ne.0) then
         call freadi('mmax=',3,mmax,0)
      end if

      nw=0
      do m1=0,mmax(1)
         do m2=0,mmax(2)
            do m3=0,mmax(3)
               nw=nw+1
            end do               
         end do               
      end do               

      if(nde.ne.0) then
         do nd=1,abs(nde)
c           read(inpu,*) (y(i,nd,1),i=1,3),icorc(nd),ilm(nd)
            call freadr('k=',3,y(1,nd,1),1)
c            call freadi('icorc=',1,icorc(nd),0)
c            call freadi('ilm=',1,ilm(nd),0)
         end do

         icom=0
         do nd=1,abs(nde)
            if(icorc(nd).eq.0) then
               icom=1
               ilm(nd)=1
            end if
         end do
c     
c        if(icom.eq.1) call freadi('ilc=',3,ilc,1)
         if(iwflag(3).eq.1) then
c     write(iout,210) nde,ied,nsymo,icent,ige,nctr
            write(iout,210) nde,nsymo,icent,ige,nctr
         end if
      end if
      if(nde.ne.0) then
         nw=nw*2  ! for cos and sin terms
         nt=1
      else
         nw=1     ! only for cos
         nt=1
      end if
      idm=3+abs(nde)+ied
      if(iwflag(3).eq.1) then
           write(iout,205)
      end if
      do nd=1,abs(nde)+ied
         if(icorc(nd).eq.1) then
            ii=1
         else if(icorc(nd).eq.0) then
            ii=2
         end if
         if(iwflag(3).eq.1) then
            write(iout,204) nd,(y(j,nd,1),j=1,3),icorc(nd),ilm(nd),
     *           fmt(ii)
         end if
      end do
      if(nde.ne.0) then
         if(iwflag(3).eq.1) then
            if(icom.eq.1) write(iout,211) ilc
            write(iout,'(1h )')
         end if
      end if
      do k=1,6
         ctr(k,0)=0
      end do
      if(nctr.gt.0) then
c        read(inpu,*) 'centring translation'
         call freadc('centering translation',0,dummy,1)
         do k=1,nctr
            read(inpu,'(a)') coord
            call decod1(coord,ctr(1,1),k)
         end do
      end if
c     read(inpu,*) 'symmetry operator'
      call freadc('symmetry operator',0,dummy,1)
      do k=1,nsymo
         read(inpu,'(a)') coord
         call decode(coord,rr,tt,k)
      end do
c*****call getri(re,ri,nsymo,y,nde)
      if(iwflag(3).eq.1) then
           if(nctr.gt.0) then
                call wtctr(ctr,nctr,idm)
           end if
           if(icent.eq.1) then
                 write(iout,2022)
           else
                 write(iout,2021)
           end if
           if(ige.eq.1) then
                write(iout,201) 'generating elements'
           end if
           write(iout,'(1h )')
      end if
      nn=abs(nde)+1
      if(iwflag(3).eq.1) then
           call wtsymo(rr,tt,nsymo,idm)
      end if
      if(ige.eq.1) then
         call ordr(rr,imps,nsymo,idm)
         do i=1,nsymo
            do j=1,idm
               tg(j,i)=tt(j,i,1)
               do  k=1,idm
                  rg(j,k,i)=rr(j,k,i,1)
               end do
            end do
         end do
         impt=1
         do i=1,6
            tt(i,1,1)=0.0
            do j=1,6
               rr(i,j,1,1)=0.0
               rr(i,i,1,1)=1.0
            end do
         end do
         do ns=1,nsymo
            imp=imps(ns)
            do i=2,imp
               do j=1,impt
                  mp1=j+(i-2)*impt
                  mp2=j+(i-1)*impt
                  call mpso(rg,tg,ns,rr,tt,mp1,mp2,idm)
               end do
            end do
            impt=impt*imp
         end do
         nsymo=impt
         if(iwflag(3).eq.1) then
            write(iout,207)
            write(iout,'(1h )')
            call wtsymo(rr,tt,nsymo,idm)
         end if
      end if
 81   continue
      do 85 n=2,nsub
          write(6,*) 'substructure',n
c         symmetry operators for the i-th subsystem
          do 86 ns=1,nsymo
              call getzrz(zmtrx(1,n),rr(1,1,ns,1),rr(1,1,ns,n))
              call getzx(zmtrx(1,n),tt(1,ns,1),tt(1,ns,n))
 86       continue
           call wtsymo(rr(1,1,1,n),tt(1,1,n),nsymo,idm)
 85   continue
      return
 100  format(7i3)
 101  format(9i3,3f9.6,i3)
 102  format(3f8.5,2i3)
 103  format(24i3)
 200  format(1h ,2x,i2,4x,9f6.2,5x,3f10.6,5x,i3)
 201  format(1h ,a)
 2021 format(1h ,'symmetry operation',10x,'noncentrosymmetric')
 2022 format(1h ,'symmetry operation',10x,'centrosymmetric')
 204  format(1h ,'wavevector k',i1,10x,3f9.6,3x,'icorc=',i3,2x
     *       ,'ilm=',i3,2x,a2,'commensurate')
 205  format(1h ,'basic wavevectors')
 206  format(1h ,'generating elements')
 207  format(1h ,'symmetry operators generated by generating elements')
 208  format(1h ,i3,10x,24i3)
 209  format(1h ,'atom',9x,'symmetry operator to be skipped =1')
c 210  format(1h ,'ndm=',i2,2x,'ied=',i2,2x,'nsymo=',i3,2x,'icent=',
 210  format(1h ,'ndm=',i2,2x,'nsymo=',i3,2x,'icent=',
     *     i2,2x,'ige=',i2,2x,'nctr=',i2)
 211  format(1h ,'periods along a,b,c axes' ,3i5)
 230  format(1h ,7i5)
      end

      subroutine getrr(rr,tt,ns,re,ri,te,ti)
      integer rr(6,6,*),re(3,3,*),ri(3,3,*)
      real tt(6,*),te(3,*),ti(3,*)
      common /cdim/ idm,nde,ied
      common /kwave/ y(3,3,3),vc(3)
      call getinp(inpu)
      call getout(iout)
      do 12 i=1,idm
      if(i.le.3) then
           tt(i,ns)=te(i,ns)
      else
           tt(i,ns)=ti(i-3,ns)
      end if
      do 12 j=1,idm
      if(i.le.3.and.j.le.3) then
           rr(i,j,ns)=re(i,j,ns)
      else if(i.le.3.and.j.gt.3) then
           rr(i,j,ns)=0
      else if(i.gt.3.and.j.le.3) then
           rr(i,j,ns)=0
           do 121 k=1,3
 121       rr(i,j,ns)=rr(i,j,ns)+y(k,i-3,1)*re(k,j,ns)
           do 122 k=1,abs(nde)
 122       rr(i,j,ns)=rr(i,j,ns)-ri(i-3,k,ns)*y(j,k,1)
      else
           rr(i,j,ns)=ri(i-3,j-3,ns)
      end if
 12   continue
      do 13 i=1,6
      do 13 j=idm+1,6
 13   rr(i,j,ns)=0
      do 14 i=idm+1,6
      do 14 j=1,6
 14   rr(i,j,ns)=0
      do 15 i=idm+1,6
 15   tt(i,ns)=0
      return
      end


      subroutine ordr(re,imps,nsymo,idm)
      integer re(6,6,*),imps(*)
      integer r(6,6),a(6,6),b(6,6)
      data eps/0.001/
      call getinp(inpu)
      call getout(iout)
      do 10 i=1,nsymo
           do 11 j=1,idm
           do 11 k=1,idm
           r(j,k)=re(j,k,i)
 11        b(j,k)=r(j,k)
           do 12 j=1,7
c    check whether r is identity operator or not
                do 16 k=1,idm
                if(r(k,k).ne.1) go to 15
                do 16 l=1,idm
                if(k.ne.l.and.r(k,l).ne.0) go to 15
 16             continue
                imps(i)=j
                go to 10
c       a=r
 15             do 13 k=1,idm
                do 13 l=1,idm
 13             a(k,l)=r(k,l)
c       r=a*b   ==> r=r*re
                do 14 k=1,idm
                do 14 l=1,idm
                r(k,l)=0
                do 14 m=1,idm
 14             r(k,l)=r(k,l)+a(k,m)*b(m,l)
 12        continue
 10   continue
      do 20 i=1,nsymo
      if(imps(i).ge.7) then
           ierror=1
           go to 21
      else
           ierror=0
      end if
 20   continue
 21   continue
      if(ierror.eq.1) then
           write(iout,'(1h ,''error in symmetry operator ?'',a,24i3)')
     *      '  order: ',(imps(i),i=1,nsymo)
      end if
      return
      end

      subroutine wtctr(ctr,nctr,idm)
      real ctr(6,0:*)
      common /cwflag/ iwflag(6)
      character*3 wctr(6,48)
      integer wd(3),w
      data wd/6,5,4/,eps/0.001/
      call getinp(inpu)
      call getout(iout)
      write(iout,'(1h )')
      write(iout,*) 'centering translation'
      call getwtt(ctr(1,1),wctr,nctr,idm)
      if(idm.eq.3) write(iout,203) ((wctr(i,k),i=1,idm),k=1,nctr)
      if(idm.eq.4) write(iout,204) ((wctr(i,k),i=1,idm),k=1,nctr)
      if(idm.eq.5) write(iout,205) ((wctr(i,k),i=1,idm),k=1,nctr)
      if(idm.eq.6) write(iout,206) ((wctr(i,k),i=1,idm),k=1,nctr)
      if(i.eq.idm) write(iout,'(1h )')
      return
 203  format(1h ,5(2(a,','),a,';   '))
 204  format(1h ,4(3(a,','),a,';   '))
 205  format(1h ,4(4(a,','),a,';   '))
 206  format(1h ,4(5(a,','),a,';   '))
      end 

      subroutine wtsymo(rr,tt,nsymo,idm)
      integer rr(6,6,*)
      real tt(6,*)
      common /cwflag/ iwflag(6)
      character*3 wtt(6,48)
      integer wd(4),w
      data wd/8,6,5,4/,eps/0.001/
      call getinp(inpu)
      call getout(iout)
      w=wd(idm-2)
      ll=(nsymo-1)/w+1
      le=nsymo-(ll-1)*w
      call wtsmop(rr,tt,nsymo,idm-3,3)
      write(iout,'(1h )')
      call getwtt(tt,wtt,nsymo,idm)
      do ii=1,ll
         ks=(ii-1)*w+1
         if(ii.ne.ll) ke=ks+w-1
         if(ii.eq.ll) ke=ks+le-1
         do i=1,idm
            if(idm.eq.3) then
               write(iout,203) ((rr(i,j,k),j=1,idm),wtt(i,k),k=ks,ke)
            else if(idm.eq.4) then
               write(iout,204) ((rr(i,j,k),j=1,idm),wtt(i,k),k=ks,ke)
            else if(idm.eq.5) then
               write(iout,205) ((rr(i,j,k),j=1,idm),wtt(i,k),k=ks,ke)
            else if(idm.eq.6) then
               write(iout,206) ((rr(i,j,k),j=1,idm),wtt(i,k),k=ks,ke)
            end if
            if(i.eq.idm) then
               write(iout,*)
            end if
         end do
      end do
      return
 203  format(9(3i2,1x,a,2x))
 204  format(8(4i2,1x,a,2x))
 205  format(7(5i2,1x,a,2x))
 206  format(6(6i2,1x,a,2x))
      end

      subroutine getwtt(tt,wtt,nsymo,idm)
      real tt(6,*)
      character*3 wtt(6,*)
      real rt(8)
      character*3 wt(8)
      data rt/0.,0.5,0.333333,0.666666,0.25,0.75,0.166666,0.833333/
      data wt/' 0 ','1/2','1/3','2/3','1/4','3/4','1/6','5/6'/
      data eps/0.00001/
      call getinp(inpu)
      call getout(iout)
      do 100 i=1,nsymo
         do 110 j=1,idm
            td=tt(j,i)
            td=mod(td,1.)
            if(td.lt.-eps) then
               td=td+1
            else if(td.gt.1-eps) then
               td=td-1
            end if
            do 120 k=1,8
               if(abs(td-rt(k)).lt.eps) then
                  if(k.eq.1) then
                     write(wtt(j,i),fmt='(i2)') int(tt(j,i))
                  else
                     wtt(j,i)=wt(k)
                  end if
                  go to 110
               end if
 120        continue
 110     continue
 100  continue
      return
      end

      subroutine mpso(r1,t1,m1,r2,t2,m2,m3,idm)
      integer r1(6,6,*),r2(6,6,48,*)
      real t1(6,*),t2(6,48,*)
      do 10 i=1,idm
         t2(i,m3,1)=0.0
         do 11 j=1,idm
            t2(i,m3,1)=t2(i,m3,1)+r1(i,j,m1)*t2(j,m2,1)
 11      continue
         t2(i,m3,1)=t2(i,m3,1)+t1(i,m1)
********************************************
         tt=t2(i,m3,1)
         t2(i,m3,1)=mod(tt,1.)
********************************************
         do 12 j=1,idm
            r2(i,j,m3,1)=0.0
            do 13 k=1,idm
               r2(i,j,m3,1)=r2(i,j,m3,1)+r1(i,k,m1)*r2(k,j,m2,1)
 13         continue
 12      continue
 10   continue
      return
      end

      subroutine rdtsym(itwin)
      common /cwflag/ iwflag(6)
      integer rrt
      common /sym1/ rrt(6,6,48,3),ttt(6,48,3)
      common /cdim/ idm,nde,ied
      character*30 coord
      character*80 dummy
      call getinp(inpu)
      call getout(iout)
c     read twin operators
c      read(inpu,*) 'twin operator'
      call freadc('twin operator',0,dummy,1)
      if(itwin.ge.1) write(iout,203) 'twin operator'
      write(iout,'(1h )')
      ie=iabs(itwin)
      do 11 i=1,ie
         j=i
         read(inpu,'(a)') coord
         call decode(coord,rrt,ttt,j)
 11   continue
      if(iwflag(3).eq.1) then
         call wtsymo(rrt(1,1,1,1),ttt(1,1,1),ie,idm)
      end if
      return
 101  format(9f5.3,3f9.6)
 201  format(1h ,2x,i2,4x,9f6.2,5x,3f10.6,5x,f10.6)
 203  format(1h ,a)
      end

      subroutine rdsymv(ja,ea,nzc,jx,ix,in)
c     ix : number of independent parameters
      common /cwflag/ iwflag(6)
      dimension ea(*),ja(4,*),ia(100),jr(100,3),er(100)
      character*4 ql,no,ia
      character*256 str
      character*80 token(100)
      data ql/' '/,no/'#'/
      call getinp(inpu)
      call getout(iout)
      if(jx.eq.0) return
      if(iwflag(3).eq.1.and.iwflag(5).gt.0) then
           write(iout,201)
      end if

      ix0=ix
      jn=in
      ja4=0
 1    continue
c     read(inpu,101) ((jr(i,j),j=1,3),er(i),ia(i),i=1,4)
      read(inpu,'(a)') str
      call gettoken(str,token,ntk)
      do jd=1,ntk
         i=(jd-1)/4+1
         j=mod(jd-1,4)+1
         is=istrln(token(jd))
         if(j.le.3) then
            read(token(jd)(1:is),'(i3)') jr(i,j)
         else if(j.eq.4) then
            if(token(jd)(is:is).ne.'+') then
               read(token(jd)(1:is),'(f10.5)') er(i)
               ia(i)=' '
            else
               read(token(jd)(1:is-1),'(f10.5)') er(i)
               ia(i)='+'
            end if
         end if
c         if(j.eq.4) then
c            write(6,*) (jr(i,l),l=1,3),er(i),ia(i)
c         end if
      end do

      ie=i

      do i=1,ie
         in=in+1
         ja4=ja4+1
         if(ia(i).eq.' ') ix=ix+1
         if(in.gt.nzc) then
            call quit('increase nzc')
         end if
         do j=1,3
            ja(j,in)=jr(i,j)
         end do
         ea(in)=er(i)
         if(ia(i).eq.' ') ja(4,ix)=ja4
         if(ia(i).eq.' ') ja4=0
         if(ia(i).eq.' '.and.ix-ix0.eq.jx) go to 15
      end do
      go to 1

 15   if(in.gt.nzc.and.ix-ix0.lt.jx) go to 1

      ix1=ix0+1
      do i=ix1,ix
         nz=ja(4,i)
         if(iwflag(3).eq.1.and.iwflag(5).gt.0) then
            write(iout,202) (no,i,(ja(j,n+jn),j=1,3),ea(n+jn),n=1,nz)
         end if
      end do

      return
 101  format(4(3i3,f10.6,a1))
 201  format(1h )
 202  format((1h ,5(a1,i3,1x,3i3,f9.4,3x)))
 203  format(1h ,'lack of nzc',i6)
      end

      subroutine rdsymw(ja,ea,nzc,jx,ix,in,isub)
      common /cwflag/ iwflag(6)
      dimension ea(*),ja(4,*),ia(4),jr(4,3),er(4),isub(*)
      common /ceat/ eat(6,3)
      common /citfs/ itfs(100)
      character*4 ia
      call getinp(inpu)
      call getout(iout)
      if(jx.eq.0) return
      if(iwflag(3).eq.1.and.iwflag(5).gt.0) then
           write(iout,201)
      end if
      ix0=ix
      jn=in
      ja4=0
 1    continue
      read(inpu,101) ((jr(i,j),j=1,3),er(i),ia(i),i=1,4)
      do 13 i=1,4
      if(ia(i).eq.' '.and.jr(i,1).eq.0.and.jr(i,2).eq.0
     *       .and.jr(i,3).eq.0) go to 13
      kend=1
      if(ia(i).eq.' ') ix=ix+1
      if(itfs(jr(i,1)).eq.0.and.jr(i,2).eq.1) kend=6
      do 30 k=1,kend
      in=in+1
      ja4=ja4+1
      if(in.gt.nzc) go to 20
      do 14 j=1,3
 14   ja(j,in)=jr(i,j)
      ea(in)=er(i)
      if(itfs(jr(i,1)).eq.0.and.jr(i,2).eq.1) ja(2,in)=k
      if(itfs(jr(i,1)).eq.0.and.jr(i,2).gt.1) ja(2,in)=jr(i,2)+5
      if(itfs(jr(i,1)).eq.0.and.jr(i,2).eq.1) then
           js=isub(ja(1,in))
           ea(in)=er(i)*eat(k,js)
      end if
 30   continue
      if(ia(i).eq.' ') ja(4,ix)=ja4
      if(ia(i).eq.' ') ja4=0
 20   if(ia(i).eq.' '.and.ix-ix0.eq.jx) go to 15
 13   continue
      go to 1
 15   if(in.gt.nzc.and.ix-ix0.lt.jx) go to 1
      if(in.gt.nzc.and.ix-ix0.eq.jx) go to 99
      ix1=ix0+1
      do 10 i=ix1,ix
      nz=ja(4,i)
      if(iwflag(3).eq.1.and.iwflag(5).gt.0) then
           write(iout,202) (i,(ja(j,n+jn),j=1,3),ea(n+jn),n=1,nz)
      end if
 10   jn=jn+nz
      return
 99   l=in-nzc
      write(iout,203) l
      stop
 101  format(4(3i3,f10.6,a1))
 201  format(1h )
 202  format((1h ,5(i3,'#',1x,3i3,f9.4,3x)))
 203  format(1h ,'lack of nzc',i6)
      end
c
      subroutine wtsmop(r1,t1,nsymo,d1,d2)
c*******************************************************************
c         representation of symmetry operators in the i.t. format
c*******************************************************************
      implicit integer (a-z)
      dimension r1(6,6,*)
      dimension r(6,6),lenc(48),ip(16)
      real t1(6,*),t(6),eps,cnt2(27,6),rtr(7)
      character xyz(6)
c        for (d2+d1) dimensional case
      character*31 z1(48),bl
      character*3 ntr(7)
      data xyz/'x','y','z','t','u','v'/
      data ntr/'1/2','1/3','2/3','1/4','3/4','1/6','5/6'/
      data rtr/0.5,0.333333,0.666666,0.25,0.75,0.166666,0.833333/
      data bl/'                            '/
      call getinp(inpu)
      call getout(iout)
      eps=0.00001
      d3=d1+d2
c******** write centering translations *********
c      if(ncnt.gt.1) then
c      do 50 i=1,ncnt
c      z1(i)=bl
c      cp=0
c      if(i.eq.1) then
c           z1(1)(1:1)='('
c           cp=1
c      end if
c      do 51 j=1,d3
c          do 52 k=1,7
c          if(abs(rtr(k)-cnt2(i,j)).lt.eps) then
c               z1(i)(cp+1:cp+3)=ntr(k)
c               cp=cp+3
c               go to 51
c          end if
c 52       continue
c          if(abs(cnt2(i,j)).lt.eps) then
c               z1(i)(cp+1:cp+1)='0'
c               cp=cp+1
c          end if
c 51   continue
c      if(i.lt.ncnt) then
c           z1(i)(cp+1:cp+1)=';'
c           cp=cp+1
c      else
c           z1(i)(cp+1:cp+1)=')'
c           cp=cp+1
c      end if
c      ip(i)=cp
c 50   continue
c      write(iout,'(1h ,6a)') (z1(i)(1:ip(i)),i=1,ncnt)
c      end if
c******** write symmetry operators *******
      do 10 ii=1,nsymo
           cp=1
           do 11 i=1,d3
           t(i)=t1(i,ii)
           do 11 j=1,d3
 11        r(i,j)=r1(i,j,ii)
           do 15 i=1,d3
                t(i)=mod(t(i),1.0)
                if(t(i).lt.eps) then
                     t(i)=t(i)+1
                else if(t(i).gt.1.0-eps) then
                     t(i)=t(i)-1
                end if
 15        continue
           do 14 i=1,22
           z1(ii)=bl
 14        continue
           do 13 i=1,d3
           do 131 j=1,7
           if(abs(t(i)-rtr(j)).le.eps) then
                z1(ii)(cp:cp+2)=ntr(j)
                cp=cp+3
                go to 132
           end if
 131       continue
           if(abs(t1(i,ii)).gt.eps) then
              if(t1(i,ii).gt.eps) then
                 write(z1(ii)(cp:cp),fmt='(i1)') int(t1(i,ii))
                 cp=cp+1
              else
                 write(z1(ii)(cp:cp+1),fmt='(i2)') int(t1(i,ii))
                 cp=cp+2
              end if
           end if
 132       continue
           do 12 j=1,d3
           if(j.le.d2) then
                k=j
           else if(j.gt.d2) then
                k=j+(3-d2)
           end if
           if(r(i,j).eq.1) then
                if(cp.eq.1) then
                     z1(ii)(cp:cp)=xyz(k)
                     cp=cp+1
                else if(z1(ii)(cp-1:cp-1).eq.',') then
                     z1(ii)(cp:cp)=xyz(k)
                     cp=cp+1
                else
                     z1(ii)(cp:cp+1)='+'//xyz(k)
                     cp=cp+2
                end if
           else if(r(i,j).eq.-1) then
                z1(ii)(cp:cp+1)='-'//xyz(k)
                cp=cp+2
           end if
           if(j.eq.d3.and.i.ne.d3) then
                z1(ii)(cp:cp)=','
                cp=cp+1
           else if(j.eq.d3.and.i.eq.d3) then
                z1(ii)(cp:cp)=';'
                cp=cp+1
           end if
 12        continue
 13        continue
           lenc(ii)=istrln(z1(ii))
 10   continue
      write(iout,'(1h ,6(a,1x))') (z1(ii)(1:lenc(ii)),ii=1,nsymo)
      return
      end

      subroutine decod1(coord,t,n)
c     centering translation in I.T into matrix representation
      character coord*30
      real t(6,*)
      common /cdim/ idm,nde,ied
      character sym(6)*20,xyz(6)
      call getinp(inpu)
      call getout(iout)
c     position of ',' 
      i=1
      do 10 k=1,idm-1
         m=index(coord(i:30),',')
         if(m.eq.0) call quit(' incorrect centering translation')
         sym(k)=coord(i:i+m-2)
         i=i+m
 10   continue
      sym(idm)=coord(i:30)
c
      do 30 j=1,idm
         t(j,n)=0
         i=index(sym(j),'/')
         if(i.gt.0) then
            read(sym(j)(i-1:i-1),'(i1)') n1
            read(sym(j)(i+1:i+1),'(i1)') n2
            t(j,n)=float(n1)/float(n2)
            if(i.ne.2.and.sym(j)(i-2:i-2).eq.'-') t(j,n)=-t(j,n)
         endif
   30 continue
      end

      subroutine decode(coord,r,t,n)
c     symmetry operation in I.T into matrix representation
      character coord*30
      integer r(6,6,*)
      real t(6,*)
      common /cdim/ idm,nde,ied
      character sym(6)*20,xyz(6)
      data xyz/'x','y','z','t','u','v'/
      call getinp(inpu)
      call getout(iout)
c     position of ',' 
      i=1
      do 10 k=1,idm-1
         m=index(coord(i:30),',')
         if(m.eq.0) call quit(' incorrect symmetry operation')
         sym(k)=coord(i:i+m-2)
         i=i+m
 10   continue
      sym(idm)=coord(i:30)
c
      do 30 j=1,idm
         t(j,n)=0
         do 20 k=1,idm
            r(j,k,n)=0
   20    continue
c
c        rotation matrix
         do 25 k=1,idm
            i=index(sym(j),xyz(k))
            if(i.gt.0) then
               if(i.eq.1) then
                  r(j,k,n)=1
               else
                  if(sym(j)(i-1:i-1).eq.'+') then
                     r(j,k,n)=1
                  else
                     r(j,k,n)=-1
                  endif
               endif
            endif
   25    continue
c
c        translation vector
         i=index(sym(j),'/')
         if(i.gt.0) then
            read(sym(j)(i-1:i-1),'(i1)') n1
            read(sym(j)(i+1:i+1),'(i1)') n2
            t(j,n)=float(n1)/float(n2)
            if(i.ne.2.and.sym(j)(i-2:i-2).eq.'-') t(j,n)=-t(j,n)
         endif
   30 continue
      end

      subroutine freadi(prfx,n,var,nl)
      integer var(*)
      character*(*) prfx
      integer ip,ep
      character*256 buf
      common /cbuf/ buf,ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      if(nl.eq.1) then
           read(inpu,'(a)',end=200) buf
           ip=1
      end if
      ipd=index(buf(ip:256),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,256
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue

          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp:256),' ')
          ipd2=index(buf(jp:256),',')
          if(ipd1.gt.0.and.ipd2.gt.0) then
              jp=min(ipd1,ipd2)+jp
          else if(ipd1.gt.0) then
              jp=ipd1+jp
          else if(ipd2.gt.0) then
              jp=ipd2+jp
          else
              jp=256
          end if
c         jp: beginning of next prefix or value
 150  continue
      if(jp.eq.256) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp),*) (var(i),i=1,n)
      ip=jp
      return
 200  continue
           write(iout,'(2a)') prfx,' not found'
           call quit(' ')
      return
      end

      subroutine freadr(prfx,n,var,nl)
      real var(*)
      character*(*) prfx
      integer ip,ep
      character*256 buf
      common /cbuf/ buf,ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      if(nl.eq.1) then
           read(inpu,'(a)',end=200) buf
           ip=1
      end if
      ipd=index(buf(ip:256),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
c      if(ipd.eq.0) then
c           write(iout,'(2a)') prfx,' not found'
c           call quit(' ')
c      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,256
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue
          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp:256),' ')
          ipd2=index(buf(jp:256),',')
          if(ipd1.gt.0.and.ipd2.gt.0) then
              jp=min(ipd1,ipd2)+jp
          else if(ipd1.gt.0) then
              jp=ipd1+jp
          else if(ipd2.gt.0) then
              jp=ipd2+jp
          else
              jp=256
          end if
c         jp: beginning of next prefix or value
 150  continue
      if(jp.eq.256) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp-1),*) (var(i),i=1,n)
      ip=jp
      return
 200  continue
          write(iout,'(2a)') prfx,' not found'
          call quit(' ')
      return
      end

      subroutine freadc(prfx,n,var,nl)
      character*(*) var(*)
      character*(*) prfx
      integer ip,ep
      character*256 buf
      common /cbuf/ buf,ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      lval=len(var(1))
      if(nl.eq.1) then
           read(inpu,fmt='(a)',end=200) buf(1:256)
           ip=1
      end if
      ipd=index(buf(ip:256),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
c      if(ipd.eq.0) then
c           write(iout,'(2a)') prfx,' not found'
c           call quit(' ')
c      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,256
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue
          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp+1:256),'''')
          if(ipd1.gt.0) then
              jp=ipd1+jp+1
          else
              jp=256
          end if
c         jp: beginning of the next prefix or value
 150  continue
      if(jp.eq.256) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp-1),*) (var(i)(1:lval),i=1,n)
      ip=jp
      return
 200  continue
           write(iout,'(2a)') prfx,' not found'
           call quit(' ')
      return
      end

      function istrln(c)
      character*(*) c
      l=len(c)
      do 10 i=l,1,-1
      if(c(i:i).ne.' ') then
          istrln=i
          return
      end if
 10   continue
      istrln=0
      return
      end

      function ioval()
      common /cwflag/ iwflag(6)
      character*6 atom
      common /cltc/ atom(100)
      common /asft/ ne,ascf(35,20),df1(20),df2(20),iand,iel(3,100)
      common /cdim/ idm,nde,ied
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      integer hkln,nslct,ih(6),hm
      integer h4,hd
      common /islct/ hd(6),h4(3,40),nslct
      entry idmf()
          idmf=idm
      return
      entry ndef()
          ndef=nde
      return
      end

      subroutine setinp(inpu)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      cinpu=inpu
      return
      end

      subroutine setout(iout)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      coutu=iout
      return
      end

      subroutine getinp(inpu)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      inpu=cinpu
      return
      end

      subroutine getout(iout)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      iout=coutu
      return
      end

      subroutine setskp(iskp)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      cskip=iskp
      return
      end

      subroutine getskp(iskp)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      iskp=cskip
      return
      end

      subroutine getzx(zmtrx,x,zx)
      integer zmtrx(*)
      real x(*),zx(*)
      do 100 i=1,6
         zx(i)=x(zmtrx(i))
 100  continue
      return
      end

      subroutine gethz(zmtrx,h,hz)
      integer zmtrx(*),h(*),hz(*)
      do i=1,6
         hz(i)=h(zmtrx(i))
      end do
      return
      end

      subroutine getzrz(zmtrx,r,zrz)
      integer zmtrx(*),r(6,*),zrz(6,*)
      do 100 i=1,6
      do 110 j=1,6
          zrz(i,j)=r(zmtrx(i),zmtrx(j))
 110      continue
 100  continue
      return
      end

      subroutine splpos(na,isk,irs,isub,mmax) 
      integer isk(0:48,*),irs(0:48,*),isub(*),mmax(*)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      integer h(6)
      integer hr(6)
      integer kstr(6,120)
      integer kind(6,120)
      integer rd(20*20*120)
      integer iskis(0:48,100)
      complex sx(90,90),sp(30,30),sb(270,270)   ! projection operators
      complex x(270),y(270),ph,ys(270,270)
      integer indx(270,270)
      common /citfs/ itfs(100)
      integer dspmdl,tmpmdl,sctmdl
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      integer getijk
      integer rs
      real ts
      common /csitesy/ rs(6,6,48,100),ts(6,48,100),nss(100)
      complex crd,getcrd
      
      complex pi2
      data pi2 /(0.,6.28318)/

c     kstar : number of kstar vectors
c     3*kstar
c     generate projection operators

      call setiskis(na)       ! calculate sitesymmetry operators
      call getiskis(iskis,na) ! its inverse operator
      call getnde(nde)
      call getidm(idm)

      ndm=abs(nde)

      do ik=1,na            ! atom loop
         if(itempf.eq.0) then
            it=0
         else if(itempf.gt.0) then
            it=1
         else
            it=itfs(ik)
         end if
         js=isub(ik)        ! js-th substructure
         ne=nss(ik)
c        write(6,*) 'site sym. op:',(irs(j,ik),j=1,ne)
         write(6,*) 'site symmetry operators'
         call wtsymo(rs(1,1,1,ik),ts(1,1,ik),ne,idm)
         call getkind(mmax,nkind,kind,js) ! get independent k
         do nk=1,nkind
            n1=kind(4,nk)
            n2=kind(5,nk)
            n3=kind(6,nk)

            call getkstr(n1,n2,n3,irs(0,ik),js,nkstr,kstr)  ! k-star
c           call getrd(nkstr,kstr,irs(0,ik),js,rd)          ! get rd
            call getrd(nkstr,kstr,irs(0,ik),ik,rd)          ! get rd

            m1=kstr(4,1)
            m2=kstr(5,1)
            m3=kstr(6,1)
            if(m1.eq.0.and.m2.eq.0.and.m3.eq.0) then
               icse=1
            else
               icse=2
            end if

            do ics=1,icse
               h(1)=0
               h(2)=0
               h(3)=0
               h(4)=m1
               h(5)=m2
               h(6)=m3
               
               if(nkstr.gt.30) then
                  call quit('nkstr > 30')
               end if
               
c              initialization
               do i=1,3*nkstr
                  do j=1,3*nkstr
                     sx(i,j)=(0.,0.)
                  end do
               end do
               do i=1,nkstr
                  do j=1,nkstr
                     sp(i,j)=(0.,0.)
                  end do
               end do
               do i=1,9*nkstr
                  do j=1,9*nkstr
                     sb(i,j)=(0.,0.)
                  end do
               end do
               
               nindx=0
               nindp=0
               nindb=0
               
               do ns=1,ne
                  nsi=iskis(ns,ik)  ! inverse operator of ns
c                 ph=cexp(pi2*vinpir(h(4),ts(4,nsi,ik),ndm))/ne
c                 ph=cexp(-pi2*vinpir(hr(4),ts(4,nsi,ik),ndm))/ne
                  ph=cexp(pi2*vinpir(h(4),ts(4,ns,ik),ndm))/ne
c                 ph=cexp(-pi2*vinpir(hr(4),ts(4,ns,ik),ndm))/ne

                  call gethrs(h,hr,ns,ik)

c                 write(6,'(27i2)') 'rs=',((rs(i,j,ns,ik),j=1,3),i=1,3)
c                 write(6,*) 'ts=',(ts(i,ns,ik),i=1,6)

                  do i=1,9*nkstr
                     do j=1,9*nkstr
                        indx(i,j)=0
                     end do
                  end do
c                  write(6,*) 'nkstr=',nkstr
                  do k=1,nkstr
                  do l=1,nkstr
c                    crd=getcrd(rd,nkstr,k,l,nsi)
                     crd=getcrd(rd,nkstr,k,l,ns)
                     cs=real(crd)
                     sn=aimag(crd)
c                     if(ns.eq.1) then
c                        write(6,*) 'k,l,cs,sn=',k,l,cs,sn
c                     end if
                     do i=1,3
                     do j=1,3
                        if(ndm.eq.0) then
                           sx(i,j)=sx(i,j)+rs(i,j,ns,ik)
                        else
                           id=i+(k-1)*3
                           jd=j+(l-1)*3
                           if(ics.eq.1) then ! cosine term
                              sx(id,jd)=sx(id,jd)+
     *                             rs(i,j,ns,ik)*cmplx(cs,0.0)*ph
                           else if(ics.eq.2) then ! sine term
                              sx(id,jd)=sx(id,jd)+
     *                             rs(i,j,ns,ik)*cmplx(0.0,sn)*ph
                           end if
                        end if
                     end do
                     end do   

c                    occupational parameters
                     if(ndm.eq.0) then
                        sp(1,1)=1
                     else
                        id=k
                        jd=l
                        if(ics.eq.1) then ! cosine term
                           sp(id,jd)=sp(id,jd)+cmplx(cs,0.0)*ph
                        else if(ics.eq.2) then ! sine term
                           sp(id,jd)=sp(id,jd)+cmplx(0.0,sn)*ph
                        end if
                     end if

                     if(it.eq.0) go to 100
c                    anisotropic thermal parameter
                     do i1=1,3
                     do i2=1,3
                     do j1=1,3
                     do j2=1,3
c                       direct product of (rs,rs~,rd)
                        id=i1+(j1-1)*3+(k-1)*9
                        jd=i2+(j2-1)*3+(l-1)*9

                        indx(id,jd)=1

                        if(ndm.eq.0) then
                           sb(id,jd)=sb(id,jd)+rs(i1,i2,ns,ik)*
     *                          rs(j2,j1,ns,ik)*cmplx(cs,0.0)
                        else
                           if(ics.eq.1) then      ! cosine term
                              sb(id,jd)=sb(id,jd)+rs(i1,i2,ns,ik)*
     *                             rs(j2,j1,ns,ik)*cmplx(cs,0.0)*ph
                           else if(ics.eq.2) then ! sine term
                              sb(id,jd)=sb(id,jd)+rs(i1,i2,ns,ik)*
     *                             rs(j2,j1,ns,ik)*cmplx(0.0,sn)*ph
                           end if
                        end if

                     end do
                     end do
                     end do
                     end do   
 100                 continue
                  end do
                  end do

c                 write(6,*) 'cs,sn',cs,sn
c                 do i=1,3*nkstr
c                    write(6,'(6f5.2)') (sx(i,j),j=1,3*nkstr)
c                 end do
               end do           ! ns loop

               call snorm(sx,90,3*nkstr)   ! normalization of sx
               do i=1,3*nkstr
                  do j=1,3*nkstr
                      y(j)=sx(j,i)*(1.0,0.0)
                  end do

                  call ynorm(y,3*nkstr)
                  if(nzerovct(y,3*nkstr).eq.1) then
c                     write(6,'(18f6.2)') (y(j),j=1,3*nkstr)
                      call indy(y,ys,nindx,3*nkstr)
                  end if
               end do

c               if(ics.eq.1) then
c                  write(6,*) 'atom',ik,' x,y,z cos term'
c               else
c                  write(6,*) 'atom',ik,' x,y,z sin term'
c               end if
c               do i=1,nindx
c                  write(6,'(18f6.2)') (ys(j,i),j=1,3*nkstr)
c               end do
               call windx(ik,ys,nkstr,nindx)

               call snorm(sp,30,nkstr)   ! normalization of sp
               do i=1,nkstr
                  do j=1,nkstr
                     y(j)=sp(j,i)*(1.0,0.0)
                  end do
                  call ynorm(y,nkstr)
                  if(nzerovct(y,nkstr).eq.1) then
c                     write(6,'(18f6.2)') (y(j),j=1,nkstr)
                      call indy(y,ys,nindp,nkstr)
                  end if
               end do

c               if(ics.eq.1) then
c                  write(6,*) 'atom',ik,' p cos term'
c               else
c                  write(6,*) 'atom',ik,' p sin term'
c               end if
c               do i=1,nindp
c                  write(6,'(18f6.2)') (ys(j,i),j=1,nkstr)
c               end do
               call windp(ik,ys,nkstr,nindp)  ! same as p

               if(it.eq.0) then
c                  if(ics.eq.1) then
c                     write(6,*) 'atom',ik,' B cos term'
c                  else
c                     write(6,*) 'atom',ik,' B sin term'
c                  end if
c                  do i=1,nindp
c                     write(6,'(18f6.2)') (ys(j,i),j=1,nkstr)
c                  end do
                  call windb(it,ik,ys,nkstr,nindp)
               end if

               if(it.eq.0) go to 110
               call snorm(sb,270,9*nkstr)   ! normalization of sb
               do i=1,9*nkstr

c                 do j=1,9*kstr
                  do ki=1,nkstr
                  do i1=1,3
                  do j2=1,3
                     j=i1+(j2-1)*3+(ki-1)*9
c                    do k=1,9*kstr
                     do kj=1,nkstr
                     do j1=1,3
                     do i2=1,3
                        k=i1+(j1-1)*3+(ki-1)*9
                        l=i2+(j2-1)*3+(kj-1)*9
                        m=i2+(j1-1)*3+(kj-1)*9
                       if(m.eq.i) then
c                       if(l.eq.i) then
                           y(j)=sb(k,l)*(1.0,0.0)
                        end if
                     end do
                     end do
                     end do
                  end do
                  end do
                  end do

                  call ynorm(y,9*nkstr)
                  if(nzerovct(y,9*nkstr).eq.1) then
c                     write(6,'(18f6.2)') (y(j),j=1,9*nkstr)
                      call indy(y,ys,nindb,9*nkstr)
                  end if
               end do

c               if(ics.eq.1) then
c                  write(6,*) 'atom',ik,' Bi cos term'
c               else
c                  write(6,*) 'atom',ik,' Bi sin term'
c               end if
c               do i=1,nindb
c                  write(6,'(18f6.2)') (ys(j,i),j=1,9*nkstr)
c               end do
               call windb(it,ik,ys,nkstr,nindb)

 110           continue
            end do         ! end ics
         end do
      end do
      if(nde.eq.0) then
         return
      else
         stop
      end if
      end

       subroutine windx(ik,ys,nkstr,nindx)
       complex ys(270,*)
       character*256 str
       data eps /0.001/, sqr3 /1.732050/
       
       do i=1,nindx
          ii=0
          do j=1,256
             str(j:j)=' '
          end do
          do j=1,nkstr
             do k=1,3
                jk=k+(j-1)*3
                ysr=real(ys(jk,i))
                ysi=aimag(ys(jk,i))
                aysr=abs(ysr)
                aysi=abs(ysi)
                if(aysr.gt.eps) then
                   if(ii.eq.0) then
                      str(ii+1:ii+1)=' '
                   else
                      str(ii+1:ii+1)='='
                   end if
                   if(ysr.gt.0) then
                      str(ii+2:ii+2)='+'
                   else
                      str(ii+2:ii+2)='-'
                   end if
                   if(abs(aysr-1).lt.eps) then
                      str(ii+3:ii+3)='1'
                   else if(abs(aysr-2).lt.eps) then
                      str(ii+3:ii+3)='2'
                   else if(abs(aysr-sqr3).lt.eps) then
                      str(ii+3:ii+3)='3'
                   else
                      call quit('invalid factor?')
                   end if
                   if(k.eq.1) then
                      str(ii+4:ii+5)='xc'
                   else if(k.eq.2) then
                      str(ii+4:ii+5)='yc'
                   else
                      str(ii+4:ii+5)='zc'
                   end if
                   if(j.le.9) then
                      write(str(ii+6:ii+6),'(i1)') j
                      ii=ii+6
                   else if(j.le.99) then
                      write(str(ii+6:ii+7),'(i2)') j
                      ii=ii+7
                   else
                      call quit('nkstr > 99')
                   end if
                end if
                if(aysi.gt.eps) then
                   if(ii.eq.0) then
                      str(ii+1:ii+1)=' '
                   else
                      str(ii+1:ii+1)='='
                   end if
                   if(ysi.gt.0) then
                      str(ii+2:ii+2)='+'
                   else
                      str(ii+2:ii+2)='-'
                   end if
                   if(abs(aysi-1).lt.eps) then
                      str(ii+3:ii+3)='1'
                   else if(abs(aysi-2).lt.eps) then
                      str(ii+3:ii+3)='2'
                   else if(abs(aysi-sqr3).lt.eps) then
                      str(ii+3:ii+3)='3'
                   else
                      call quit('invalid factor?')
                   end if
                   if(k.eq.1) then
                      str(ii+4:ii+5)='xs'
                   else if(k.eq.2) then
                      str(ii+4:ii+5)='ys'
                   else
                      str(ii+4:ii+5)='zs'
                   end if
                   if(j.le.9) then
                      write(str(ii+6:ii+6),'(i1)') j
                      ii=ii+6
                   else if(j.le.99) then
                      write(str(ii+6:ii+7),'(i2)') j
                      ii=ii+7
                   else
                      call quit('nkstr > 99')
                   end if
                end if
             end do
          end do
          istr=istrln(str)
          write(6,'(a,i3,a)') 'atom ',ik, str(1:istr)
       end do
       return
       end

       subroutine windb(it,ik,ys,nkstr,nindb)
       complex ys(270,*)
       character*256 str
       data eps /0.001/, sqr3 /1.732050/
       do i=1,nindb
          ii=0
          do j=1,256
             str(j:j)=' '
          end do
          do j=1,nkstr
             if(it.eq.1) then
                do k=1,3
                do l=k,3
c               do l=1,3
                   jk=l+(k-1)*3+(j-1)*9
                   ysr=real(ys(jk,i))
                   ysi=aimag(ys(jk,i))
                   aysr=abs(ysr)
                   aysi=abs(ysi)
                   if(aysr.gt.eps) then
                      if(ii.eq.0) then
                         str(ii+1:ii+1)=' '
                      else
                         str(ii+1:ii+1)='='
                      end if
                      if(ysr.gt.0) then
                         str(ii+2:ii+2)='+'
                      else
                         str(ii+2:ii+2)='-'
                      end if
                      if(abs(aysr-1).lt.eps) then
                         str(ii+3:ii+3)='1'
                      else if(abs(aysr-2).lt.eps) then
                         str(ii+3:ii+3)='2'
                      else if(abs(aysr-sqr3).lt.eps) then
                         str(ii+3:ii+3)='3'
                      else
                         call quit('invalid factor?')
                      end if
                      if(k.eq.1.and.l.eq.1) then
                         str(ii+4:ii+6)='b1c'
                      else if(k.eq.2.and.l.eq.2) then
                         str(ii+4:ii+6)='b2c'
                      else if(k.eq.3.and.l.eq.3) then
                         str(ii+4:ii+6)='b3c'
                      else if(k.eq.2.and.l.eq.3) then
                         str(ii+4:ii+6)='b4c'
                      else if(k.eq.3.and.l.eq.2) then
                         str(ii+4:ii+6)='b4c'
                      else if(k.eq.1.and.l.eq.3) then
                         str(ii+4:ii+6)='b5c'
                      else if(k.eq.3.and.l.eq.1) then
                         str(ii+4:ii+6)='b5c'
                      else if(k.eq.1.and.l.eq.2) then
                         str(ii+4:ii+6)='b6c'
                      else if(k.eq.2.and.l.eq.1) then
                         str(ii+4:ii+6)='b6c'
                      end if
                      if(j.le.9) then
                         write(str(ii+7:ii+7),'(i1)') j
                         ii=ii+7
                      else if(j.le.99) then
                         write(str(ii+7:ii+8),'(i2)') j
                         ii=ii+8
                      else
                         call quit('nkstr > 99')
                      end if
                   end if
                   if(aysi.gt.eps) then
                      if(ii.eq.0) then
                         str(ii+1:ii+1)=' '
                      else
                         str(ii+1:ii+1)='='
                      end if
                      if(ysi.gt.0) then
                         str(ii+2:ii+2)='+'
                      else
                         str(ii+2:ii+2)='-'
                      end if
                      if(abs(aysi-1).lt.eps) then
                         str(ii+3:ii+3)='1'
                      else if(abs(aysi-2).lt.eps) then
                         str(ii+3:ii+3)='2'
                      else if(abs(aysi-sqr3).lt.eps) then
                         str(ii+3:ii+3)='3'
                      else
                         call quit('invalid factor?')
                      end if
                      if(k.eq.1.and.l.eq.1) then
                         str(ii+4:ii+6)='b1s'
                      else if(k.eq.2.and.l.eq.2) then
                         str(ii+4:ii+6)='b2s'
                      else if(k.eq.3.and.l.eq.3) then
                         str(ii+4:ii+6)='b3s'
                      else if(k.eq.2.and.l.eq.3) then
                         str(ii+4:ii+6)='b4s'
                      else if(k.eq.3.and.l.eq.2) then
                         str(ii+4:ii+6)='b4s'
                      else if(k.eq.1.and.l.eq.3) then
                         str(ii+4:ii+6)='b5s'
                      else if(k.eq.3.and.l.eq.1) then
                         str(ii+4:ii+6)='b5s'
                      else if(k.eq.1.and.l.eq.2) then
                         str(ii+4:ii+6)='b6s'
                      else if(k.eq.2.and.l.eq.1) then
                         str(ii+4:ii+6)='b6s'
                      end if
                      if(j.le.9) then
                         write(str(ii+7:ii+7),'(i1)') j
                         ii=ii+7
                      else if(j.le.99) then
                         write(str(ii+7:ii+8),'(i2)') j
                         ii=ii+8
                      else
                         call quit('nkstr > 99')
                      end if
                   end if
                end do
                end do
             else if(it.eq.0) then
                jk=j
                ysr=real(ys(jk,i))
                ysi=aimag(ys(jk,i))
                aysr=abs(ysr)
                aysi=abs(ysi)
                if(aysr.gt.eps) then
                   if(ii.eq.0) then
                      str(ii+1:ii+1)=' '
                   else
                      str(ii+1:ii+1)='='
                   end if
                   if(ysr.gt.0) then
                      str(ii+2:ii+2)='+'
                   else
                      str(ii+2:ii+2)='-'
                   end if
                   if(abs(aysr-1).lt.eps) then
                      str(ii+3:ii+3)='1'
                   else if(abs(aysr-2).lt.eps) then
                      str(ii+3:ii+3)='2'
                   else if(abs(aysr-sqr3).lt.eps) then
                      str(ii+3:ii+3)='3'
                   else
                      call quit('invalid factor?')
                   end if
                   str(ii+4:ii+5)='bc'
                   if(j.le.9) then
                      write(str(ii+6:ii+6),'(i1)') j
                      ii=ii+6
                   else if(j.le.99) then
                      write(str(ii+6:ii+7),'(i2)') j
                      ii=ii+7
                   else
                      call quit('nkstr > 99')
                   end if
                else if(aysi.gt.eps) then
                   if(ii.eq.0) then
                      str(ii+1:ii+1)=' '
                   else
                      str(ii+1:ii+1)='='
                   end if
                   if(ysi.gt.0) then
                      str(ii+2:ii+2)='+'
                   else
                      str(ii+2:ii+2)='-'
                   end if
                   if(abs(aysi-1).lt.eps) then
                      str(ii+3:ii+3)='1'
                   else if(abs(aysi-2).lt.eps) then
                      str(ii+3:ii+3)='2'
                   else if(abs(aysi-sqr3).lt.eps) then
                      str(ii+3:ii+3)='3'
                   else
                      call quit('invalid factor?')
                   end if
                   str(ii+4:ii+5)='bs'
                   if(j.le.9) then
                      write(str(ii+6:ii+6),'(i1)') j
                      ii=ii+6
                   else if(j.le.99) then
                      write(str(ii+6:ii+7),'(i2)') j
                      ii=ii+7
                   else
                      call quit('nkstr > 99')
                   end if
                end if
             end if
          end do
          istr=istrln(str)
          if(istr.ge.1) then
             write(6,'(a,i3,a)') 'atom ',ik, str(1:istr)
          end if
       end do
       return
       end

       subroutine windp(ik,ys,nkstr,nindp)
       complex ys(270,*)
       character*256 str
       data eps /0.001/, sqr3 /1.732050/
       do i=1,nindp
          ii=0
          do j=1,256
             str(j:j)=' '
          end do
          do j=1,nkstr
             jk=j
             ysr=real(ys(jk,i))
             ysi=aimag(ys(jk,i))
             aysr=abs(ysr)
             aysi=abs(ysi)
             if(aysr.gt.eps) then
                if(ii.eq.0) then
                   str(ii+1:ii+1)=' '
                else
                   str(ii+1:ii+1)='='
                end if
                if(ysr.gt.0) then
                   str(ii+2:ii+2)='+'
                else
                   str(ii+2:ii+2)='-'
                end if
                if(abs(aysr-1).lt.eps) then
                   str(ii+3:ii+3)='1'
                else if(abs(aysr-2).lt.eps) then
                   str(ii+3:ii+3)='2'
                else if(abs(aysr-sqr3).lt.eps) then
                   str(ii+3:ii+3)='3'
                else
                   call quit('invalid factor?')
                end if
                str(ii+4:ii+5)='pc'
                if(j.le.9) then
                   write(str(ii+6:ii+6),'(i1)') j
                   ii=ii+6
                else if(j.le.99) then
                   write(str(ii+6:ii+7),'(i2)') j
                   ii=ii+7
                else
                   call quit('nkstr > 99')
                end if
             end if
             if(aysi.gt.eps) then
                if(ii.eq.0) then
                   str(ii+1:ii+1)=' '
                else
                   str(ii+1:ii+1)='='
                end if
                if(ysi.gt.0) then
                   str(ii+2:ii+2)='+'
                else
                   str(ii+2:ii+2)='-'
                end if
                if(abs(aysi-1).lt.eps) then
                   str(ii+3:ii+3)='1'
                else if(abs(aysi-2).lt.eps) then
                   str(ii+3:ii+3)='2'
                else if(abs(aysi-sqr3).lt.eps) then
                   str(ii+3:ii+3)='3'
                else
                   call quit('invalid factor?')
                end if
                str(ii+4:ii+5)='ps'
                if(j.le.9) then
                   write(str(ii+6:ii+6),'(i1)') j
                   ii=ii+6
                else if(j.le.99) then
                   write(str(ii+6:ii+7),'(i2)') j
                   ii=ii+7
                else
                   call quit('nkstr > 99')
                end if
             end if
          end do
          istr=istrln(str)
          write(6,'(a,i3,a)') 'atom ',ik, str(1:istr)
       end do
       return
       end

      subroutine getkij(k,i,j)
c     integer kij(2,6)
      integer kij(2,9)
c      data kij /1,1,2,2,3,3,2,3,3,1,1,2/
      data kij /1,1,1,2,1,3,2,1,2,2,2,3,3,1,3,2,3,3/
      i=kij(1,k)
      j=kij(2,k)
      return
      end

      integer function getijk(i,j)
      integer ijk(3,3)
c     11 21 31 12 22 32 13 23 33
      data ijk /1,6,5,6,2,4,5,4,3/
      getijk=ijk(i,j)
      return
      end

      subroutine indy(y,ys,nind,n)
      complex y(*),ys(270,*)
      complex yi(270)
      do i=1,nind
         if(icompcn(y,ys(1,i),n).eq.1) then
            return
         end if
      end do
      do i=1,n
         yi(i)=-y(i)
      end do
      do i=1,nind
         if(icompcn(yi,ys(1,i),n).eq.1) then
            return
         end if
      end do

      nind=nind+1
      do i=1,n
         ys(i,nind)=y(i)
      end do
      return
      end

      subroutine ynorm(y,nkstr)   ! normalization of y
      complex y(*)
      real smin
      data eps /0.001/
      smin=0
      do i=1,nkstr
         abss1=abs(real(y(i)))
         abss2=abs(aimag(y(i)))
         if(abss1.gt.eps) then
            if(smin.eq.0) then
               smin=abss1
            else if(abss1.lt.smin) then
               smin=abss1
            end if
         end if
         if(abss2.gt.eps) then
            if(smin.eq.0) then
               smin=abss2
            else if(abss2.lt.smin) then
               smin=abss2
            end if
         end if
      end do
      
      if(smin.gt.eps) then
c     write(6,*) 'y before normalization',smin
c     write(6,'(18f6.2)') (y(i),i=1,nkstr)
         do i=1,nkstr
            y(i)=y(i)/smin
         end do
c     write(6,*) 'y'
c     write(6,'(18f6.2)') (y(i),i=1,nkstr)
      end if
      return
      end

      subroutine snorm(s,n1,nkstr)   ! normalization of s
      complex s(n1,*)  ! projection operators
      real smin
      data eps /0.0001/

      smin=0
      do i=1,nkstr
         do j=1,nkstr
            abss1=abs(real(s(i,j)))
            abss2=abs(aimag(s(i,j)))
            if(abss1.gt.eps) then
               if(smin.eq.0) then
                  smin=abss1
               else if(abss1.lt.smin) then
                  smin=abss1
               end if
            end if
            if(abss2.gt.eps) then
               if(smin.eq.0) then
                  smin=abss2
               else if(abss2.lt.smin) then
                  smin=abss2
               end if
            end if
         end do
      end do

      if(smin.lt.eps) then
         return
      end if

      do i=1,nkstr
         do j=1,nkstr
           s(i,j)=s(i,j)/smin
         end do
      end do

c      write(6,*) 'propjection operator'
c      do i=1,nkstr
c        write(6,'(27f4.1)') (s(i,j),j=1,nkstr)
c      end do
      return
      end

      subroutine getrd(nkstr,kstr,irs,ik,rd)
      integer rd(nkstr,nkstr,*)
      integer kstr(6,*)
      integer irs(0:48)   ! site symmetry operator for ik atom
      integer hr(6),hs(6)

      call getnsym(nsymo)

      do n=1,irs(0)
c        ns=irs(n)
         do i=1,nkstr
            do j=1,nkstr
               rd(i,j,n)=0
            end do
         end do

         do i=1,nkstr
            call gethrs(kstr(1,i),hr,n,ik)
c           call getjs(js)  ! get subsystem
c           call gethr(kstr(1,i),hr,ns,ic,js)
            do j=1,6
               hs(j)=-hr(j)
            end do
            do j=1,nkstr
               if(icomp(kstr(1,j),hr).eq.1) then
                  rd(i,j,n)=1
                  go to 100
               else if(icomp(kstr(1,j),hs).eq.1) then
                  rd(i,j,n)=-1
                  go to 100
               end if
            end do
            call quit('rd invalid')
 100        continue
         end do

c         write(6,*) 'rd='
c         do i=1,nkstr
c            write(6,'(27i2)') (rd(i,j,n),j=1,nkstr)
c         end do
      end do
      return
      end

      complex function getcrd(rd,nkstr,k,l,ns)
      integer rd(nkstr,nkstr,*)
      getcrd=cmplx(abs(rd(k,l,ns)),rd(k,l,ns))

c      if(k.eq.1.and.l.eq.1) then
c         write(6,*) ' '
c         do i=1,nkstr
c            write(6,'(27i2)') (rd(i,j,ns),j=1,nkstr)
c         end do
c      end if
      return
      end

      subroutine getkstr(m1,m2,m3,irs,js,nkstr,kstr)
      integer m1,m2,m3
      integer irs(0:48)
      integer h(6),hr(6),hr0(6)
      integer kstr(6,*)

      call getidm(idm)
      call getnsym(nsymo)
c     write(6,*) 'kstar'
      nkstr=0
      do n=1,irs(0)
         ns=irs(n)
         if(ns.gt.nsymo) then
            ns=ns-nsymo
         end if
         h(1)=0
         h(2)=0
         h(3)=0
         h(4)=m1
         h(5)=m2
         h(6)=m3
         call gethr(h,hr0,ns,0,js)
         if(nkstr.gt.0) then
            do ic=0,1
               call gethr(hr0,hr,1,ic,js)
               do nk=1,nkstr
                  if(icomp(hr,kstr(1,nk)).eq.1) go to 120
               end do
            end do
         end if
         nkstr=nkstr+1
         call vcpyi(hr0,kstr(1,nkstr),idm)
         do j=idm+1,6
            kstr(j,nkstr)=0
         end do
c         write(6,*) (kstr(k,nkstr),k=4,6)
 120     continue
      end do
      write(6,'(a,6(a,i2,a,3i3,a))') 'kstar',
     *     (' #',nk,' (',(kstr(k,nk),k=4,6),') ',nk=1,nkstr)
      return
      end

      subroutine getkind(mmax,nkind,kind,js)
      integer kind(6,*),mmax(*)
      integer h(6),hr(6)
      call getnsym(nsymo)
      call geticnt(icent)
      call getidm(idm)
      nkind=0
      do m3=0,mmax(3)
         do m2=0,mmax(2)
            do m1=0,mmax(1)
               h(1)=0
               h(2)=0
               h(3)=0
               h(4)=m1
               h(5)=m2
               h(6)=m3
               if(nkind.gt.0) then
                  do ns=1,nsymo
                     do ic=0,icent
                        call gethr(h,hr,ns,ic,js)
                        do nk=1,nkind
                           if(icomp(hr,kind(1,nk)).eq.1) go to 120
                        end do
                     end do
                  end do
               end if
               nkind=nkind+1
               call vcpyi(h,kind(1,nkind),idm)
               do j=idm+1,6
                  kind(j,nkind)=0
               end do
 120           continue
            end do
         end do
      end do
c      do nk=1,nkind
c         write(6,*) 'kind=',(kind(j,nk),j=4,6)
c      end do
      return
      end

      function nzerovct(y,n)
c     real y(*)
      complex y(*)
      data eps /0.0001/
      do i=1,n
         if(abs(y(i)).gt.eps) then
            nzerovct=1
            return
         end if
      end do
      nzerovct=0
      return
      end

      subroutine sitesy(na,mpl,x,isk,irs,isub) 
      real mpl(*),x(6,*)
      integer isk(0:48,*),irs(0:48,*),isub(*)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      common /cdim/ idm,nde,ied
      real rx(6)
      real rxs(6,48)
      integer rcomp
      integer rs
      real ts
      common /csitesy/ rs(6,6,48,100),ts(6,48,100),nss(100)
      real td(6)

      do 50 jk=1,na
         is=isub(jk)
         nsym=0
         nrx=1
         isk(nrx,jk)=1
         call savex(x(1,jk),rxs(1,nrx))
         do 130 ic=0,icent
            do 100 ns=1,nsymo
               call getrx(x(1,jk),rx,rr(1,1,1,is),tt(1,1,is),ns,ic)
               do irx=1,nrx
                  if(rcomp(rx,rxs(1,irx),is,td).eq.1) go to 120
               end do
               nrx=nrx+1        ! equivalent positions
               call savex(rx,rxs(1,nrx))
               if(ic.eq.0) then
                  isk(nrx,jk)=ns
               else
                  isk(nrx,jk)=ns+nsymo
               end if
 120           continue
               if(rcomp(rx,x(1,jk),is,td).eq.1) then ! site symmetry op.
                  nsym=nsym+1
                  if(ic.eq.0) then
                     irs(nsym,jk)=ns
                  else
                     irs(nsym,jk)=ns+nsymo
                  end if
                  if(ic.eq.0) then
                     do i=1,6
                        do j=1,6
                           rs(i,j,nsym,jk)=rr(i,j,ns,is)
                        end do
                     end do
                     do i=1,6
                        ts(i,nsym,jk)=tt(i,ns,is)+td(i)
                     end do
                  else
                     do i=1,6
                        do j=1,6
                           rs(i,j,nsym,jk)=-rr(i,j,ns,is)
                        end do
                     end do
                     do i=1,6
                        ts(i,nsym,jk)=-tt(i,ns,is)-td(i)
                     end do
                  end if
c                  write(6,*) 'rs0=',((rs(i,j,nsym,jk),j=1,3),i=1,3)
               end if
 100        continue
 130     continue
         isk(0,jk)=nrx
         irs(0,jk)=nsym
         nss(jk)=nsym
         mpl(jk)=1
         if(icent.eq.1) then
c     compress symmetry element > nsymo
            n=0
            do 200 j=1,isk(0,jk)
               if(isk(j,jk).gt.nsymo) go to 201
               n=n+1
 200        continue
 201        continue
c     n: number of equivalent positions generated by given operators
c     isk(0,jk): number of equivalent positions generated by given
c     symmetry operators and inversion
            mpl(jk)=float(isk(0,jk))/(float(n)*2)
            isk(0,jk)=n
         end if
 50   continue
      write(6,*) 'site symmetry group'
      do jk=1,na
         write(6,'(a,i3,a,(20i3))') 'atom',jk,'  ',
     *        (irs(j,jk),j=1,irs(0,jk))
      end do

      write(6,*) 'multiplicity  equivalent position'
      do 56 jk=1,na
         write(6,*) 'atom',jk,mpl(jk),(isk(j,jk),j=1,isk(0,jk))
 56   continue
      
      return
      end

      subroutine savex(rx,rxs)
      real rx(*),rxs(*)
      common /cdim/ idm,nde,ied
      do i=1,idm
         rxs(i)=rx(i)
      end do
      return
      end

      integer function rcomp(x,rx,is,td)
      real x(*),rx(*),td(*)
      common /cdim/ idm,nde,ied
      common /cctr/ ctr(6,0:48),nctr
      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      real tt(6)
      data eps/0.00001/
      rcomp=0
      do 110 j=0,nctr
         call getzx(zmtrx(1,is),ctr(1,j),tt)
         do 100 i=1,3
            xx=abs(x(i)-rx(i)-tt(i))+eps/2
            xx=mod(xx,1.)
            if(xx.gt.eps) go to 110
 100     continue

         do i=1,6
            td(i)=tt(i)
         end do
c         write(6,*) 'td0=',(td(i),i=1,6)
         rcomp=1
         return
 110  continue
      return
      end

      subroutine getrx(x,rx,r,t,ns,ic)
      real x(*),rx(*),t(6,*)
      integer r(6,6,*)
      common /cdim/ idm,nde,ied
      do 100 i=1,idm
           rx(i)=0
           do 101 j=1,idm
                rx(i)=rx(i)+r(i,j,ns)*x(j)
 101       continue
           rx(i)=rx(i)+t(i,ns)
           if(ic.eq.1) then
               rx(i)=-rx(i)
           end if
 100  continue
      return
      end

      subroutine gethr(h,hr,ns,ic,js)
      integer h(*),hr(*)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      common /cdim/ idm,nde,ied
      do i=1,idm
         hr(i)=0
         do j=1,idm
            hr(i)=hr(i)+h(j)*rr(j,i,ns,js)
         end do
         if(ic.eq.1) then
            hr(i)=-hr(i)
         end if
      end do
      return
      end

      subroutine gethrs(h,hr,ns,ik)
      integer h(*),hr(*)
      integer rs
      real ts
      common /csitesy/ rs(6,6,48,100),ts(6,48,100),nss(100)
      common /cdim/ idm,nde,ied
      do i=1,idm
         hr(i)=0
         do j=1,idm
            hr(i)=hr(i)+h(j)*rs(j,i,ns,ik)
         end do
      end do
      return
      end

      subroutine getyl(hr,yl,js)
      integer hr(*)
      real yl(*)
      common /kwave/ y(3,3,3),vc(3)
      do j=1,3
         yl(j)=hr(j)+hr(4)*y(j,1,js)+hr(5)*y(j,2,js)+
     *        hr(6)*y(j,3,js)
      end do
      return
      end
      
      function getyt(jh,ns,js)
      integer jh(*)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      getyt=jh(1)*tt(1,ns,js)+jh(2)*tt(2,ns,js)+jh(3)*tt(3,ns,js)+
     *      jh(4)*tt(4,ns,js)+jh(5)*tt(5,ns,js)+jh(6)*tt(6,ns,js)
      return
      end 

      subroutine gethrt(h,hr,ns)
      integer h(*),hr(*)
      integer rrt
      common /sym1/ rrt(6,6,48,3),ttt(6,48,3)
      common /cdim/ idm,nde,ied
c     for twin
      do i=1,idm
         hr(i)=0
         do j=1,idm
            hr(i)=hr(i)+h(j)*rrt(j,i,ns,1)
         end do
      end do
      do j=1,6
         hr(j)=hr(j)+ttt(j,ns,1)
      end do
      return
      end


      subroutine gauss1
      implicit real*8 (a-h,o-z)
      real pxj,hj
      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      dimension p(3)
c     calculation of interval and weight of gauss integral
      do 12 l=1,3
         n=ilm(l)
         if(icorc(l).eq.1) then
c           incommensurate case
            eps=1.0d-10
            do 10 k=1,ilm(l)
               xm=float(n+1-2*k)/float(2*n+1)
               x=dsin(xm*3.1415926536)
 30            p(1)=1.0
               p(2)=x
               do  i=2,n
                  p(3)=x*p(2)+float(i-1)/float(i)*(x*p(2)-p(1))
                  p(1)=p(2)
                  p(2)=p(3)
               end do
               d=(1.0d0-x**2)*p(2)/(float(n)*(p(1)-x*p(2)))
               if(dabs(d).lt.eps) go to 40
               x=x-d
               go to 30
 40            w=2.0d0*(1.0d0-x**2)/(float(n)*p(1))**2
               pxj(k,l)=-x/2.0d0
               hj(k,l)=w/2.0d0
 10         continue
*****************
c            write(6,'(a,10e13.5)') 'pxj=',(pxj(k,l),k=1,ilm(l))
c            write(6,'(a,10e13.5)') 'hj =',(hj(k,l),k=1,ilm(l))
*****************
         else
c     commensurate case
            do k=1,ilm(l)
               pxj(k,l)=float(k-1)/float(n)
               hj(k,l)=1.0/float(n)
            end do
         end if
 12   continue
      return
      end

      function vinp(x,y,n)
c*****inner product of n-dimensional vactors, x,y
      real x(*),y(*)
      vinp=0
      do 10 i=1,n
 10   vinp=vinp+x(i)*y(i)
      return
      end

      function vabs(x,n)
c*****length of an n-dimensional vactor x
      dimension x(*)
      vabs=0
      do 10 i=1,n
 10   vabs=vabs+x(i)*x(i)
      vabs=sqrt(vabs)
      return
      end

      subroutine getipnf(ip1,ip2,ip3)
      common /ctlint3/ ipnf1,ipnf2,ipnf3
      ip1=ipnf1
      ip2=ipnf2
      ip3=ipnf3
      return
      end

      subroutine setiwg(iwg)
      common /ciwg/ iwgs
      iwgs=iwg
      return
      end 

      subroutine getiwg(iwg)
      common /ciwg/ iwgs
      iwg=iwgs
      return
      end 

      subroutine getgij(zmtrx,nsub)
      integer zmtrx(6,*)
c       get metric tensor for each substructure and
c       get wave vector of modulation wave for each substructure
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *        as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /mtrtno/ ax(3,3,3),ay(3,3,3)
      common /kwave/ y(3,3,3),vc(3)
      common /cdim/ idm,nde,ied
      real bb(3),piv(3)
      integer ipiv(3),indx(3,2)
      real bx(3,6)
      data bx/1,0,0,
     *        0,1,0,
     *        0,0,1,
     *        0,0,0,
     *        0,0,0,
     *        0,0,0/

      sina=sqrt(1-cosa**2)
      sinb=sqrt(1-cosb**2)
      sinc=sqrt(1-cosc**2)
      gabx(1,1)=a*sqrt(1.0-cosa**2-cosb**2-cosc**2+2.0*cosa*cosb*cosc)/
     *         sina
      gabx(1,2)=a*(cosc-cosa*cosb)/sina
      gabx(1,3)=a*cosb
      gabx(2,1)=0.0
      gabx(2,2)=b*sina
      gabx(2,3)=b*cosa
      gabx(3,1)=0.0
      gabx(3,2)=0.0
      gabx(3,3)=c
      vc0=gabx(1,1)*gabx(2,2)*gabx(3,3)

      do 100 i=1,3
      do 100 j=1,3
 100  gaby(j,i)=gabx(i,j)

      call matinv(gaby,3,3,bb,0,det,ipiv,indx,piv)
c      write(6,'(a,9f10.4)') 'gabx=',((gabx(i,j),j=1,3),i=1,3)
c      write(6,'(a,9f10.4)') 'gaby=',((gaby(i,j),j=1,3),i=1,3)
      call yyy(bx(1,1))
      call yyy(bx(1,2))
      call yyy(bx(1,3))

      do 200 n=1,abs(nde)
          do 210 j=1,3
          bx(j,n+3)=y(j,n,1)
 210      continue
          call yyy(bx(1,n+3))
 200  continue
c                 for k-th subsustem
c      ax(i,j,k) : i-th components of j-th unit vector in the direct space
c                   in orthogonal coordinate system
c      ay(i,j,k) : i-th components of j-th unit vector in the reciprocal space
c                   in orthogonal coordinate system
       Do 300 n=1,nsub
          do 310 id=1,3
          i=zmtrx(id,n)
          ax(id,1,n)=bx(1,i)
          ax(id,2,n)=bx(2,i)
          ax(id,3,n)=bx(3,i)
          ay(id,1,n)=bx(1,i)
          ay(id,2,n)=bx(2,i)
          ay(id,3,n)=bx(3,i)
 310      continue
          call matinv(ax(1,1,n),3,3,bb,0,det,ipiv,indx,piv)
c         vc(n)=1/det(ax)
c      vc : unit cell volume for n-th substructure
          vc(n)=1/det

          do 320 id=1,3
          do 320 jd=1,3
              i=zmtrx(id,n)
              j=zmtrx(jd,n)
              gabn(id,jd,n)=vinp(bx(1,i),bx(1,j),3)
              gab(id,jd,n)=gabn(id,jd,n)
 320      continue
          call matinv(gab(1,1,n),3,3,bb,0,det,ipiv,indx,piv)

c           write(6,*) 'ax=',((ax(i,j,n),i=1,3),j=1,3)
c           write(6,*) 'ay=',((ay(i,j,n),i=1,3),j=1,3)
c         ax: a,b,c for n-th substructure (orthogonal system)
          an=vabs(ax(1,1,n),3)
          bn=vabs(ax(1,2,n),3)
          cn=vabs(ax(1,3,n),3)
          cosan=vinp(ax(1,2,n),ax(1,3,n),3)/(bn*cn)
          cosbn=vinp(ax(1,3,n),ax(1,1,n),3)/(cn*an)
          coscn=vinp(ax(1,1,n),ax(1,2,n),3)/(an*bn)
          if(nsub.gt.1) then
              write(6,*) 'substructure',n
              write(6,400) an,bn,cn,cosan,cosbn,coscn
          end if
          do 330 kd=1,abs(nde)
              k=zmtrx(kd+3,n)
              y(1,kd,n)=vinp(ax(1,1,n),bx(1,k),3)
              y(2,kd,n)=vinp(ax(1,2,n),bx(1,k),3)
              y(3,kd,n)=vinp(ax(1,3,n),bx(1,k),3)
              if(nsub.gt.1) then
                  write(6,*) ' k=',y(1,kd,n),y(2,kd,n),y(3,kd,n)
              end if
 330      continue
 300  continue
      as=sqrt(gabn(1,1,1))
      bs=sqrt(gabn(2,2,1))
      cs=sqrt(gabn(3,3,1))
      cosas=gabn(2,3,1)/(bs*cs)
      cosbs=gabn(1,3,1)/(as*cs)
      coscs=gabn(1,2,1)/(as*bs)
      sinas=sqrt(1.0-cosas**2)
      sinbs=sqrt(1.0-cosbs**2)
      sincs=sqrt(1.0-coscs**2)
      return
 400  format(' a=',f9.4,1x,'b=',f9.4,1x,'c=',f9.4,1x,
     *    'cosa=',f9.5,1x,'cosb=',f9.5,1x,'cosc=',f9.5)
      end

      subroutine getgijs(zmtrx,nsub)
      integer zmtrx(6,*)
c       get metric tensor for each substructure and
c       get wave vector of modulation wave for each substructure
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *        as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /mtrtno/ ax(3,3,3),ay(3,3,3)
      common /kwave/ y(3,3,3),vc(3)
      common /cdim/ idm,nde,ied
      real bb(3),piv(3)
      integer ipiv(3),indx(3,2)
      real bx(3,6)
      data bx/1,0,0,
     *        0,1,0,
     *        0,0,1,
     *        0,0,0,
     *        0,0,0,
     *        0,0,0/
      sinas=sqrt(1-cosas**2)
      sinbs=sqrt(1-cosbs**2)
      sincs=sqrt(1-coscs**2)
      gaby(1,1)=as
      gaby(1,2)=0.0
      gaby(1,3)=0.0
      gaby(2,1)=bs*coscs
      gaby(2,2)=bs*sincs
      gaby(2,3)=0
      gaby(3,1)=cs*cosbs
      gaby(3,2)=cs*(cosas-cosbs*coscs)/sincs
      gaby(3,3)=cs*sqrt(1.0-cosas**2-cosbs**2-coscs**2+
     *     2.0*cosas*cosbs*coscs)/sincs
      vc0=gaby(1,1)*gaby(2,2)*gaby(3,3)
      do i=1,3
         do j=1,3
            gabx(j,i)=gaby(i,j)
         end do
      end do
      call matinv(gabx,3,3,bb,0,det,ipiv,indx,piv)
c      write(6,'(a,9f10.4)') 'gabx=',((gabx(i,j),j=1,3),i=1,3)
c      write(6,'(a,9f10.4)') 'gaby=',((gaby(i,j),j=1,3),i=1,3)
      call yyy(bx(1,1))
      call yyy(bx(1,2))
      call yyy(bx(1,3))
      do n=1,abs(nde)
         do j=1,3
            bx(j,n+3)=y(j,n,1)
            continue
         end do
         call yyy(bx(1,n+3))
      end do
c                 for k-th subsustem
c      ax(i,j,k) : i-th components of j-th unit vector in the direct space
c                   in orthogonal coordinate system
c      ay(i,j,k) : i-th components of j-th unit vector in the reciprocal space
c                   in orthogonal coordinate system
       Do 300 n=1,nsub
          do id=1,3
             i=zmtrx(id,n)
             ax(id,1,n)=bx(1,i)
             ax(id,2,n)=bx(2,i)
             ax(id,3,n)=bx(3,i)
             ay(id,1,n)=bx(1,i)
             ay(id,2,n)=bx(2,i)
             ay(id,3,n)=bx(3,i)
             continue
          end do
          call matinv(ax(1,1,n),3,3,bb,0,det,ipiv,indx,piv)
c         vc(n)=1/det(ax)
c      vc : unit cell volume for n-th substructure
          vc(n)=1/det
          do id=1,3
             do jd=1,3
                i=zmtrx(id,n)
                j=zmtrx(jd,n)
                gabn(id,jd,n)=vinp(bx(1,i),bx(1,j),3)
                gab(id,jd,n)=gabn(id,jd,n)
                
             end do
          end do
          call matinv(gab(1,1,n),3,3,bb,0,det,ipiv,indx,piv)
c           write(6,*) 'ax=',((ax(i,j,n),i=1,3),j=1,3)
c           write(6,*) 'ay=',((ay(i,j,n),i=1,3),j=1,3)
c         ax: a,b,c for n-th substructure (orthogonal system)
          an=vabs(ax(1,1,n),3)
          bn=vabs(ax(1,2,n),3)
          cn=vabs(ax(1,3,n),3)
          cosan=vinp(ax(1,2,n),ax(1,3,n),3)/(bn*cn)
          cosbn=vinp(ax(1,3,n),ax(1,1,n),3)/(cn*an)
          coscn=vinp(ax(1,1,n),ax(1,2,n),3)/(an*bn)
          if(nsub.eq.1.or.n.eq.1) then
             a=an
             b=bn
             c=cn
             cosa=cosan
             cosb=cosbn
             cosc=coscn
             write(6,400) a,b,c,cosa,cosb,cosc
          else if(nsub.gt.1) then
             write(6,*) 'substructure',n
             write(6,400) an,bn,cn,cosan,cosbn,coscn
          end if
          do 330 kd=1,nde
              k=zmtrx(kd+3,n)
              y(1,kd,n)=vinp(ax(1,1,n),bx(1,k),3)
              y(2,kd,n)=vinp(ax(1,2,n),bx(1,k),3)
              y(3,kd,n)=vinp(ax(1,3,n),bx(1,k),3)
              if(nsub.gt.1) then
                  write(6,*) ' k=',y(1,kd,n),y(2,kd,n),y(3,kd,n)
              end if
 330      continue
 300  continue
      return
 400  format(' a=',f9.4,1x,'b=',f9.4,1x,'c=',f9.4,1x,
     *    'cosa=',f9.5,1x,'cosb=',f9.5,1x,'cosc=',f9.5)
      end

      subroutine xxx(r)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      dimension rr(3),r(3)
      do 10 i=1,3
      rr(i)=0.0
      do 10 j=1,3
      rr(i)=rr(i)+gabx(i,j)*r(j)
 10   continue
      r(1)=rr(1)
      r(2)=rr(2)
      r(3)=rr(3)
      return
      end

      subroutine yyy(r)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      dimension rr(3),r(3)
      do 10 i=1,3
      rr(i)=0.0
      do 10 j=1,3
      rr(i)=rr(i)+r(j)*gaby(j,i)
 10   continue
      r(1)=rr(1)
      r(2)=rr(2)
      r(3)=rr(3)
      return
      end

      subroutine xxxs(r,isub)
      common /mtrtno/ gabx(3,3,3),gaby(3,3,3)
      dimension rr(3),r(3)
      do 10 i=1,3
      rr(i)=0.0
      do 10 j=1,3
      rr(i)=rr(i)+gabx(i,j,isub)*r(j)
 10   continue
      r(1)=rr(1)
      r(2)=rr(2)
      r(3)=rr(3)
      return
      end

      subroutine yyys(r,isub)
      common /mtrtno/ gabx(3,3,3),gaby(3,3,3)
      dimension rr(3),r(3)
      do 10 i=1,3
      rr(i)=0.0
      do 10 j=1,3
      rr(i)=rr(i)+r(j)*gaby(j,i,isub)
 10   continue
      r(1)=rr(1)
      r(2)=rr(2)
      r(3)=rr(3)
      return
      end

      subroutine calu1(icase,as,bs,cs,cosas,cosbs,coscs,ud)
      real ud(3,3)  ! first 3x3 part of u matrix
      sinas=sqrt(1.-cosas**2)
      sinbs=sqrt(1.-cosbs**2)
      sincs=sqrt(1.-coscs**2)
      d=sqrt(1.0-cosas**2-cosbs**2-coscs**2+2.0*cosas*cosbs*coscs)
      if(icase.eq.1) then
         ud(1,1)=as
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=bs*coscs
         ud(2,2)=bs*sincs
         ud(2,3)=0.0
         ud(3,1)=cs*cosbs
         ud(3,2)=cs*(cosas-cosbs*coscs)/sincs
         ud(3,3)=cs*d/sincs
      else if(icase.eq.2) then
         ud(1,1)=bs
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=cs*cosas
         ud(2,2)=cs*sinas
         ud(2,3)=0.0
         ud(3,1)=as*coscs
         ud(3,2)=as*(cosbs-coscs*cosas)/sinas
         ud(3,3)=as*d/sinas
      else if(icase.eq.3) then
         ud(1,1)=cs
         ud(1,2)=0.0
         ud(1,3)=0.0
         ud(2,1)=as*cosbs
         ud(2,2)=as*sinbs
         ud(2,3)=0.0
         ud(3,1)=bs*cosas
         ud(3,2)=bs*(coscs-cosas*cosbs)/sinbs
         ud(3,3)=bs*d/sinbs
      else
         call quit('icase should be 1,2 or 3')
      end if
      return
      end

      subroutine getltcs(ad,bd,cd,cosad,cosbd,coscd)
      common /ltccnt/ a,b,c,cosa,cosb,cosc,sina,sinb,sinc,
     *        as,bs,cs,cosas,cosbs,coscs,sinas,sinbs,sincs
      ad=as
      bd=bs
      cd=cs
      cosad=cosas
      cosbd=cosbs
      coscd=coscs
      return
      end

      subroutine calltcs(a,b,c,cosa,cosb,cosc,as,bs,cs,cosas,
     *     cosbs,coscs)
      real gabs(3,3),bd(3),piv(3)
      integer ip(3),indx(6)
      gabs(1,1)=a*a
      gabs(2,2)=b*b
      gabs(3,3)=c*c
      gabs(1,2)=a*b*cosc
      gabs(1,3)=a*c*cosb
      gabs(2,3)=b*c*cosa
      gabs(2,1)=gabs(1,2)
      gabs(3,1)=gabs(1,3)
      gabs(3,2)=gabs(2,3)
      call matinv(gabs,3,3,bd,0,det,ip,indx,piv) !  matrix inversion of gabs
      as=sqrt(gabs(1,1))
      bs=sqrt(gabs(2,2))
      cs=sqrt(gabs(3,3))
      cosas=gabs(2,3)/bs/cs
      cosbs=gabs(3,1)/cs/as
      coscs=gabs(1,2)/as/bs
      return
      end

      subroutine setu1(u1)
      real u1(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            ui(i,j)=u1(i,j)
         end do
      end do
      return
      end

      subroutine setu2(u2)
      real u2(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            ui(i+3,j)=u2(i,j)
         end do
      end do
      return
      end

      subroutine getu1(u1)
      real u1(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            u1(i,j)=ui(i,j)
         end do
      end do
      return
      end

      subroutine getu2(u2)
      real u2(3,3)
      common /cui/ ui(6,3)
      do i=1,3
         do j=1,3
            u2(i,j)=ui(i+3,j)
         end do
      end do
      return
      end

      subroutine setui(uid)
      real uid(6,3)
      common /cui/ ui(6,3)
      do i=1,6
         do j=1,3
            ui(i,j)=uid(i,j)
         end do
      end do
      return
      end

      subroutine getui(uid)
      real uid(6,3)
      common /cui/ ui(6,3)
      do i=1,6
         do j=1,3
            uid(i,j)=ui(i,j)
         end do
      end do
      return
      end

      subroutine getmu(mu1)
      real mu, mu1, numin, numax
      common /cphi/ phimin, phimax, rotmin, rotmax, ang0, mthd, mu,
     *     rc, ps, cc, z0, zl, ar, layer, numin, numax
      mu1=mu
      return
      end

      subroutine setmu(mu1)
      real mu, mu1, numin, numax
      common /cphi/ phimin, phimax, rotmin, rotmax, ang0, mthd, mu,
     *     rc, ps, cc, z0, zl, ar, layer, numin, numax
      mu=mu1
      return
      end

      subroutine cylnd(x,y,z,rx,ry,iskip)
      real x,y,z,rx,ry
      integer iskip, mthd
      real mu,nu
      common /ccylnd/ r,s
c     x : x  in cartesian coordinate of the diffraction vector
c     y : y  in cartesian coordinate
c     z : z  in cartesian coordinate
c     rx: angle  in cylindorical coordinate
c     ry: radius in cylindorical coordinate

c     r: effective radius for the layer
c     s: effective radius for the zero layer
c     nu: nu angle
      data eps/0.000001/
c
      call getmthd(mthd)
      call getmu(mu)
      rx=atan3(y,x)             ! -pi < rx <= pi  (x is parallel to X-ray)
      ry=sqrt(x**2+y**2)        ! radius 
      if(mthd.eq.1) then        ! equi-inclination setting
         if(abs(z/2).gt.1-eps) then
            iskip=1
            return
         end if
c        mu is calculated
         nu=asin(z/2)
         mu=nu
         r=cos(mu)
         s=cos(mu)
      else if(mthd.eq.2.or.mthd.eq.3) then ! inclined-beam setting
c        mu is given 
         if(abs(z-sin(mu)).gt.1-eps) then
            iskip=2
            return
         end if         
         nu=asin(z-sin(mu))
         r=cos(nu)
         s=cos(mu)
      end if
      if(ry.lt.abs(s-r)+eps) then
         iskip=3
         return
      else if(ry.gt.s+r-eps) then
         iskip=4
         return
      end if
      iskip=0
      return
      end

      subroutine trnsp3(sm)
      real smd(3,3),sm(3,3)
      do 100 i=1,3
      do 100 j=1,3
           smd(i,j)=sm(j,i)
 100  continue
      do 110 i=1,3
      do 110 j=1,3
           sm(i,j)=smd(i,j)
 110  continue
      return
      end

      subroutine setomt(phi,theta,psi)
      common /comtrx/ omtrx1(3,3),omtrx2(3,3),omtrx3(3,3),
     *     euang1(3),euang2(3),euang3(3)
      call setomtr(phi,theta,psi,omtrx1)
      return
      end

      subroutine getomt(omtrx)
      real omtrx(3,3)
      common /comtrx/ omtrx1(3,3),omtrx2(3,3),omtrx3(3,3),
     *     euang1(3),euang2(3),euang3(3)
      call mcpymnr(omtrx1,omtrx,3,3)
      return
      end

      subroutine weis2(rx,ry,z,sx,sy,sz,rota,icond)
      real rx,ry,z,sx(2),sy(2),sz(2),rota(2)
      integer icond
c     phi angle calculation in diffraction condition
c       for a given diffraction vector there two positions or none
c       in the Weissenberg method.
c       if no solution icond=-1 else 0
c       
c     transformation of rx, ry into px, py
c     rx: angle in cylindorical coordinate
c     ry: radius in cylindorical coordinate (in 1/lambda)
c     z:  z-coordinate
c     r : radius of circle at z on Evald shpere
c         (z : z coordinate of cylindorical coordinate)
c     s : radius of circle at z=0 on Evald shpere
c             return values for two positions
c     sx,sy,sz: normal vector of the outward x-ray (diffracted beam)
c     rota: rotation angle in the diffraction condition
c
      real mu,phi,arg1,arg2,px,py,r,s
      data eps/0.000001/
      data pi /3.1415926/
      data pi2/6.2831853/
c     arg : angle on the film in radian

      call getrs(r,s)
      call getmu(mu)
      call getrotm(rotmin,rotmax)
      icond=-1
      if(z.gt.1-eps) then
         return
      end if

      arg1=(r**2+s**2-ry**2)/(2*r*s)   ! r**2+s**2-2*r*s*cos(arg)=ry**2
      if(arg1.lt.1-eps) then 
         phi=acos(arg1)      ! 0 <= phi <= pi
      else
         phi=0               ! no solution
         return
      end if

      arg2=(ry**2+s**2-r**2)/(2*ry*s)  ! ry**2+s**2-2*ry*s*cos(arg)=r**2
      if(arg2.lt.1-eps) then
         rxd=acos(arg2)      ! 0 <= rxd <= pi
      else
         rxd=0.              ! no diffraction condition exists
         return
      end if
      px=pi-rxd     ! +px, -px : angles under the relection conditions
c                   ! from x-axis   (0 < px < pi)

c     first position
      sx(1)=r*cos(phi)   ! unit vector on a circle with radius r
      sy(1)=r*sin(phi)
      sz(1)=z-sin(mu)    ! z=sin(mu)+sin(nu) and sz=sin(nu)

      rota(1)=mod(px-rx,pi2)  ! sample rotation angle for the first position
c      if(rota(1).gt.rotmax.and.rota(1)-pi2.lt.rotmax) then
c         rota(1)=rota(1)-pi2
c      else if(rota(1).lt.rotmin) then
c         rota(1)=rota(1)+pi2
c      end if

c     second position
      sx(2)=r*cos(phi)
      sy(2)=-r*sin(phi)
      sz(2)=z-sin(mu)

      rota(2)=mod(-px-rx,pi2)  ! sample rotation angle for the second position
c      if(rota(2).gt.rotmax.and.rota(2)-pi2.lt.rotmax) then
c         rota(2)=rota(2)-pi2
c      else if(rota(2).lt.rotmin) then
c         rota(2)=rota(2)+pi2
c      end if
      icond=0
      return
      end

      subroutine setomt1(phi,theta,psi)
      common /comtrx/ omtrx1(3,3),omtrx2(3,3),omtrx3(3,3),
     *     euang1(3),euang2(3),euang3(3)
      call setomtr(phi,theta,psi,omtrx1)
      return
      end

      subroutine getomt1(omtrx)
      real omtrx(3,3)
      common /comtrx/ omtrx1(3,3),omtrx2(3,3),omtrx3(3,3),
     *     euang1(3),euang2(3),euang3(3)
      call mcpymnr(omtrx1,omtrx,3,3)
      return
      end

      function atan3(y,x)
      real x,y
      data pi /3.1415926536/
      data eps /0.00001/
      if(abs(x).lt.eps) then
         if(y.gt.eps) then
            atan3=pi/2
         else
            atan3=-pi/2
         end if
      else
         atan3=atan2(y,x)    ! -pi < atan3 <= pi
      end if
      return
      end

      subroutine setrs(r1,s1)
      common /ccylnd/ r,s
      r=r1
      s=s1
      return
      end

      subroutine getrs(r1,s1)
      common /ccylnd/ r,s
      r1=r
      s1=s
      return
      end

      subroutine setomtr(phi,theta,psi,omtrx)
      real omtrx(3,3)
      omtrx(1,1)= cos(psi)*cos(phi)-cos(theta)*sin(phi)*sin(psi)
      omtrx(1,2)= cos(psi)*sin(phi)+cos(theta)*cos(phi)*sin(psi)
      omtrx(1,3)= sin(psi)*sin(theta)
      omtrx(2,1)=-sin(psi)*cos(phi)-cos(theta)*sin(phi)*cos(psi)
      omtrx(2,2)=-sin(psi)*sin(phi)+cos(theta)*cos(phi)*cos(psi)
      omtrx(2,3)= cos(psi)*sin(theta)
      omtrx(3,1)= sin(theta)*sin(phi)
      omtrx(3,2)=-sin(theta)*cos(phi)
      omtrx(3,3)= cos(theta)
      return
      end

      subroutine getrotm(rot1,rot2)
      real mu, numin, numax
      common /cphi/ phimin, phimax, rotmin, rotmax, ang0, mthd, mu,
     *     rc, ps, cc, z0, zl, ar, layer, numin, numax
      rot1=rotmin
      rot2=rotmax
      return
      end

      subroutine mcpymnr(x,y,m,n)
      real x(m,n),y(m,n)
      do i=1,m
         do j=1,n
            y(i,j)=x(i,j)
         end do
      end do
      return
      end

      subroutine mcpymni(x,y,m,n)
      integer x(m,n),y(m,n)
      do i=1,m
         do j=1,n
            y(i,j)=x(i,j)
         end do
      end do
      return
      end

      subroutine getmthd(mthd1)
      real mu
      common /cextcr/ mthd,avds,avma,mu
      mthd1=mthd
      return
      end

      subroutine setmthd(mthd1)
      common /cextcr/ mthd,avds,avma,mu
      mthd=mthd1
      return
      end

      subroutine getavds(avdsd)
      real mu
      common /cextcr/ mthd,avds,avma,mu
      avdsd=avds
      return
      end

      subroutine setavds(avdsd)
      common /cextcr/ mthd,avds,avma,mu
      avds=avdsd
      return
      end

      subroutine getavma(avmad)
      real mu
      common /cextcr/ mthd,avds,avma,mu
      avmad=avma
      return
      end

      subroutine setavma(avmad)
      real mu
      common /cextcr/ mthd,avds,avma,mu
      avma=avmad
      return
      end

      subroutine wtequiv(ihkl,fobs,wcor)
      integer ihkl(6)
      real lf
      call getnde(nde)
      if(nde.eq.0) then
         write(6,'(3i4,2f10.2,6f10.5)')
     *        (ihkl(k),k=1,3),fobs,wcor
      else if(nde.eq.1) then
         write(6,'(4i4,2f10.2,6f10.5)')
     *        (ihkl(k),k=1,4),fobs,wcor
      else if(nde.eq.2) then
         write(6,'(5i4,2f10.2,6f10.5)')
     *        (ihkl(k),k=1,5),fobs,wcor
      else if(nde.eq.3) then
         write(6,'(6i4,2f10.2,6f10.5)')
     *        (ihkl(k),k=1,6),fobs,wcor
      end if
      return
      end

      subroutine vcpyi(dd,de,n)
      integer dd(*),de(*)
      do j=1,n
         de(j)=dd(j)
      end do
      return
      end

      subroutine qsort(x,st,ip,nx)
c     quick sort (ascending order of x)
c     nx: the number of data x
c     ip: the initial order
c     st: a work array
      real x(nx)
      integer r,s,st(nx,0:1),ip(nx)
      do 10 i=1,nx
 10   ip(i)=i
      s=1
      st(1,0)=1
      st(1,1)=nx
    1 l=st(s,0)
      r=st(s,1)
      s=s-1
    2 i=l
      j=r
      xt=x((l+r)/2)
    3 if(i.lt.nx) then
         if(x(i).lt.xt) then
            i=i+1
            go to 3
         endif
      endif
    4 if(j.gt.1) then
         if(xt.lt.x(j)) then
            j=j-1
            go to 4
         endif
      endif
      if(i.le.j) then
        temp=x(j)
        x(j)=x(i)
        x(i)=temp
        itemp=ip(j)
        ip(j)=ip(i)
        ip(i)=itemp
        if(i.le.nx.and.j.ge.1) then
           i=i+1
           j=j-1
           go to 3
        end if
      endif
      if(j-l.le.r-i) go to 5
      if(i.lt.r) then
         s=s+1
         st(s,0)=i
         st(s,1)=r
      endif
      r=j
      go to 6
    5 if(l.lt.j) then
         s=s+1
         st(s,0)=l
         st(s,1)=j
      endif
      l=i
    6 if(l.lt.r) go to 2
      if(s.ne.0) go to 1
      return
      end

      subroutine getnsyic(nsy,ic)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      nsy=nsymo
      ic=icent
      return
      end

      subroutine setitwn(itwind)
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      itwin=itwind
      return
      end

      subroutine getitwn(itwind)
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      itwind=itwin
      return
      end

      subroutine getitmp(itmpf)
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      itmpf=itempf
      return
      end

      subroutine setiski
      common /ciski/ iski(0:48)
      integer r(6,6)
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      iski(0)=nsymo
      do i=1,nsymo
         do j=1,nsymo
            call mtmpli(rr(1,1,i,1),rr(1,1,j,1),r)
            if(identm(r).eq.1) then
               iski(i)=j
               go to 200
            end if
         end do
         call quit('error in invisk')
 200     continue
      end do
      do i=1,nsymo
         iski(i+nsymo)=iski(i)+nsymo
      end do
c      write(6,*) 'inverse operators'
c      write(6,'(a,20i3/(5x,20i3))') 'iski=',
c     *     (iski(j),j=1,nsymo*(1+abs(icent)))
      return
      end

      subroutine getiskis(iskisd,na)
      integer iskisd(0:48,*)
      common /ciskis/ iskis(0:48,100)
      integer r(6,6)
      integer rs
      real ts
      common /csitesy/ rs(6,6,48,100),ts(6,48,100),nss(100)
      common /cdim/ idm,nde,ied
      do jk=1,na
         nsym=iskis(0,jk)
         call vcpyi(iskis(0,jk),iskisd(0,jk),nsym+1)
      end do
      return
      end

      subroutine setiskis(na)
      common /ciskis/ iskis(0:48,100)
      integer r(6,6)
      integer rs
      real ts
      common /csitesy/ rs(6,6,48,100),ts(6,48,100),nss(100)
      common /cdim/ idm,nde,ied
      do jk=1,na
         nsymo=nss(jk)
         iskis(0,jk)=nsymo
         do i=1,nsymo
            do j=1,nsymo
               call mtmpli(rs(1,1,i,jk),rs(1,1,j,jk),r)
               if(identm(r).eq.1) then
                  iskis(i,jk)=j
                  go to 200
               end if
            end do
            call quit('error in setiskis')
 200        continue
         end do
c         write(6,*) 'inverse operators'
c         write(6,'(a,20i3/(5x,20i3))') 'iskis=',
c     *        (iskis(j,jk),j=1,nsymo)
      end do
      return
      end

      subroutine mtmpli(a,b,c)
      integer a(6,6),b(6,6),c(6,6)
      call getidm(idm)
      do i=1,idm
         do j=1,idm
            c(i,j)=0.
            do k=1,idm
               c(i,j)=c(i,j)+a(i,k)*b(k,j)
            end do
         end do
      end do
      return
      end

      function identm(rr)
c     if rr is identity matrix identm=1 else 0
      integer rr(6,6)
      call getidm(idm)
      identm=0
      do i=1,idm
         do j=1,idm
            if(i.eq.j) then
               if(rr(i,i).ne.1) return
            else
               if(rr(i,j).ne.0) return
            end if
         end do
      end do
      identm=1
      return
      end

      subroutine getiski(iskid)
      integer iskid(0:48)
      common /ciski/ iski(0:48)
      integer r
      common /sym/ r(6,6,48,3),t(6,48,3),nsymo,icent
      do i=0,nsymo*(1+icent)
         iskid(i)=iski(i)
      end do
      return
      end

      subroutine prmtbl(x,df,isub,ips,na,nw,ja,ea,bx,nc,iw)
      real x(*),df(*),ea(*)
      integer isub(*),ips(*),ja(4,*)
      character*(*) bx(na,nc,*)
c     common /cdim/ idm,nde
      common /cdim/ idm,nde,ied
      character*6 atom
      common /cltc/ atom(100)
      common /const/ pi2,i7,i8,i9,i10,i11,i12,i13,i14
      common /jobctl/ dspmdl,tmpmdl,sctmdl,itempf,itwin
      common /cctwn/ dt(6,48)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /citfs/ itfs(100)

      character*4 comp1(3),comp2(0:12),comp3(1),comp4(7),comp5(3),
     *            comp6(0:12)
      real wx(10,2,100),wy(10,2,100),zx(10,2,100)
      integer dspmdl,tmpmdl,sctmdl
      real x0(6,100),p0(100),b0(6,100)
      real x0w(3,100),p0w(100),b0w(6,100)
      character*4 x0d(3,100),p0d(100),b0d(6,100)
      character*4 aa,wc(20)

c****************************************************************
c    write atomic parameter table
c    iw=1: parameter table, iw=2: standard dev. table
c    iw=3: atomic parameters for a superstructure
c****************************************************************
      call getix(ixpb)
      call getnu(nu)
c      write(6,*) 'ixpb,na,nw=', ixpb,na,nw
c      write(6,'(6f10.5)') (x(i),i=1,ixpb)

      if(itempf.eq.0) then
         write(6,*) 'atom   ft    x         y         z         p',
     *        '        B'
      else if(itempf.eq.1) then
         write(6,*) 'atom   ft    x         y         z         pC',
     *        '        Beq'
         write(6,*) '             B1        B2        B3        B4',
     *        '       B5        B6'
      else if(itempf.eq.2) then
         write(6,*) 'atom   ft    x         y         z         p',
     *        '        Ueq'
         write(6,*) '             U1        U2        U3        U4',
     *        '       U5        U6'
      else if(itempf.eq.-1) then
         write(6,*) 'atom    ft    x         y         z         p',
     *        '        Beq'
         write(6,*) '              B1        B2        B3        B4',
     *        '        B5        B6'
      else if(itempf.eq.-2) then
         write(6,*) 'atom    ft     x         y        z         p',
     *        '        Ueq'
        write(6,*) '                U1        U2       U3        U4',
     *        '        U5        U6'
      end if

      do i=1,na
         jk=isub(i)
         do jd=1,nw/2
            do k=1,10
               wx(k,1,jd)=0
               wx(k,2,jd)=0
               wy(k,1,jd)=0
               wy(k,2,jd)=0
               zx(k,1,jd)=0
               zx(k,2,jd)=0
            end do
         end do
         do j=1,nw
            jd=(j-1)/2+1  ! 1 1 2 2 3 3 for j=1 2 3 4 5 6
            if(j.eq.1) then
               if(itempf.eq.0) then
                  ke=5
               else if(itempf.gt.0) then
                  ke=10
               else
                  if(itfs(i).eq.0) then
                     ke=5
                  else
                     ke=10
                  end if
               end if
            else
               if(itempf.eq.0) then
                  ke=10
               else if(itempf.gt.0) then
                  ke=20
               else
                  if(itfs(i).eq.0) then
                     ke=10
                  else
                     ke=20
                  end if
               end if
            end if
            if(iw.eq.1.or.iw.eq.2) then ! diff or std
               in=0
               do ix=1,ixpb     ! independent parameter number
                  nz=ja(4,ix)
                  do n=1,nz
                     in=in+1
                     if(ja(1,in).eq.i.and.ja(3,in).eq.j) then
                        k=ja(2,in)           ! x,y,z,p,B/B1 (U/U1) etc.
c                        write(6,'(a,3i3,f8.5)')
c     *                       'i,j,k,ea=',i,j,k,ea(in)
                        ad=x(ix)*ea(in)
                        if(j.eq.1) then
                           wx(k,1,jd)=wx(k,1,jd)+ad
                        else if(mod(j,2).eq.1) then
                           wx(k,1,jd)=wx(k,1,jd)+ad
                        else if(mod(j,2).eq.0) then
                           wx(k,2,jd)=wx(k,2,jd)+ad
                        end if                           
                        if(iw.gt.0) then
                           bd=df(ix)*ea(in)
                        end if

c                       write(6,*) 'i,j,k,jd=',i,j,k,jd,wx(k,1,jd)

                        if(iw.eq.1) then
                                ! parameters and their differences
                           if(j.eq.1) then
                              wy(k,1,jd)=wy(k,1,jd)+bd
                           else if(mod(j,2).eq.1) then
                              wy(k,1,jd)=wy(k,1,jd)+bd
                           else if(mod(j,2).eq.0) then
                              wy(k,2,jd)=wy(k,2,jd)+bd
                           end if
                        else if(iw.eq.2) then
                                ! parameters and their std.
                           if(j.eq.1) then
                              zx(k,1,jd)=zx(k,1,jd)+bd**2
                              wy(k,1,jd)=sqrt(zx(k,1,jd))
                           else if(mod(j,2).eq.1) then
                              zx(k,1,jd)=zx(k,1,jd)+bd**2
                              wy(k,1,jd)=sqrt(zx(k,1,jd))
                           else if(mod(j,2).eq.0) then
                              zx(k,2,jd)=zx(k,2,jd)+bd**2
                              wy(k,2,jd)=sqrt(zx(k,2,jd))
                           end if
                        end if
                     end if
                  end do
               end do
            end if
         end do

c        write(6,*) ' '
         jk=isub(i)
c           call getbeq(x,df,ix,ea,in,ad,bd,ja2,jk)
c           call getueq(x,df,ix,ea,in,ad,bd,ja2,jk)
c         do j=1,nw/2
c            write(6,'(10f8.5)') (wx(k,1,j),k=1,5),(wx(k,2,j),k=1,5)
c         end do

         do j=1,nw/2
            jd=(j-1)*2
            if(j.eq.1) then
               if(itempf.eq.0.or.(itempf.lt.0.and.itfs(i).eq.0)) then
                  write(6,'(a,i3,1x,4(f8.5,a,1x),1(f8.4,a,1x))')
     *                 atom(i),j,(wx(k,1,j),bx(i,k,jd+1),k=1,5)
               else
                  write(6,'(a,i3,1x,4(f8.5,a,1x),6(f6.4,a,1x),$)')
     *                 atom(i),j,(wx(k,1,j),bx(i,k,jd+1),k=1,4)
                  write(6,'(10x,4(f8.5,a,1x),6(f8.4,a,1x))')
     *                           (wx(k,1,j),bx(i,k,jd+1),k=5,10)
               end if
            else if(j.gt.1) then
               if(itempf.eq.0.or.(itempf.lt.0.and.itfs(i).eq.0)) then
                  write(6,'(a,i3,1x,4(f8.5,a,1x),1(f8.4,a,1x),$)')
     *                 atom(i),j,(wx(k,1,j),bx(i,k,jd+1),k=1,5)
                  write(6,'(1x,4(f8.5,a,1x),1(f8.4,a,1x))')
     *                           (wx(k,2,j),bx(i,k,jd+2),k=1,5)
               else
                  write(6,'(a,i3,1x,4(f8.5,a,1x),$)')
     *                 atom(i),j,(wx(k,1,j),bx(i,k,jd+1),k=1,4)
                  write(6,'(20x,4(f8.5,a,1x))')
     *                 (wx(k,2,j),bx(i,k,jd+2),k=1,4)
                  write(6,'(9x,6(f8.4,a,1x),$)')
     *                 (wx(k,1,j),bx(i,k,jd+1),k=5,10)
                  write(6,'(1x,6(f8.4,a,1x))')
     *                 (wx(k,2,j),bx(i,k,jd+2),k=5,10)
               end if
            end if
         end do
      end do
      if(itwin.ne.0) then
         call setvolr0(itwin)
      end if
 40   continue
      return
 200  format(1h ,a,1x,a,6(i6,'(',a,')'))
 203  format(1h ,11x,   6(i6,'(',a,')'))
 202  format(1h ,'fourier trm',6(i6,6x))
 204  format(1h ,'           ',6(i6,6x))
      end

      subroutine getbeq(x,df,ix,ea,in,ad,bd,ja2,jk)
      real x(*),df(*),ea(*)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      as=sqrt(gabn(1,1,jk))
      bs=sqrt(gabn(2,2,jk))
      cs=sqrt(gabn(3,3,jk))
      if(ja2.eq.1) then
         ad= x(ix)*ea(in)*4/3*as*bs/100
         bd=df(ix)*ea(in)*4/3*as*bs/100
      else if(ja2.eq.2) then
         ad= x(ix)*ea(in)*4/3*bs*bs/100
         bd=df(ix)*ea(in)*4/3*bs*bs/100
      else if(ja2.eq.3) then
         ad= x(ix)*ea(in)*4/3*cs*cs/100
         bd=df(ix)*ea(in)*4/3*cs*cs/100
      else if(ja2.eq.4) then
         ad= x(ix)*ea(in)*8/3*bs*cs/100
         bd=df(ix)*ea(in)*8/3*bs*cs/100
      else if(ja2.eq.5) then
         ad= x(ix)*ea(in)*8/3*cs*as/100
         bd=df(ix)*ea(in)*8/3*cs*as/100
      else if(ja2.eq.6) then
         ad= x(ix)*ea(in)*8/3*as*bs/100
         bd=df(ix)*ea(in)*8/3*as*bs/100
      end if
      return
      end

      subroutine getueq(x,df,ix,ea,in,ad,bd,ja2,jk)
      real x(*),df(*),ea(*)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      a=sqrt(gab(1,1,jk))
      b=sqrt(gab(2,2,jk))
      c=sqrt(gab(3,3,jk))
      as=sqrt(gabn(1,1,jk))
      bs=sqrt(gabn(2,2,jk))
      cs=sqrt(gabn(3,3,jk))
      if(ja2.eq.1) then
         ad= x(ix)*ea(in)/3*a*a*as*as
         bd=df(ix)*ea(in)/3*a*a*as*as
      else if(ja2.eq.2) then
         ad= x(ix)*ea(in)/3*b*b*bs*bs
         bd=df(ix)*ea(in)/3*b*b*bs*bs
      else if(ja2.eq.3) then
         ad= x(ix)*ea(in)/3*c*c*cs*cs
         bd=df(ix)*ea(in)/3*c*c*cs*cs
      else if(ja2.eq.4) then
         ad= x(ix)*ea(in)*2/3*b*c*bs*cs
         bd=df(ix)*ea(in)*2/3*b*c*bs*cs
      else if(ja2.eq.5) then
         ad= x(ix)*ea(in)*2/3*c*a*cs*as
         bd=df(ix)*ea(in)*2/3*c*a*cs*as
      else if(ja2.eq.6) then
         ad= x(ix)*ea(in)*2/3*a*b*as*bs
         bd=df(ix)*ea(in)*2/3*a*b*as*bs
      end if
      return
      end

      subroutine wx(x,nx,iw)
      dimension x(*),k(9)
      character iw(*)
      common /const/ pi2,j(8)
      character*4 no
      data no/'#'/
      ie=j(8)
      write(6,205) (no,i,x(i),iw(i),i=1,ie)
      write(6,*) 
 205  format((1h ,8(a1,i3,1x,f8.5,a1,1x)))
      return
      end

      subroutine getisext(isextd)
      common /ctlint1/ nx,nzc,nl,isext,nxmax
      isextd=isext
      return
      end

      subroutine quit(str)
      character*(*) str
      real tarray(2)
      write(6,*) str
c      time=etime(tarray)
c      write(6,*) '******elapsed time=',time,'sec******'
      stop
      end

      subroutine matinv(a,nm,n,b,m,determ,ipivot,index,pivot)
c
c    n*n matrix a is replaced by its inverse matrix a**-1
c    if m is nonzero vector b is replaced by a**-1*b
c
      dimension a(nm,nm),b(nm),ipivot(nm),index(nm,2),pivot(nm)
      equivalence (irow,jrow),(icolum,jcolum),(amax,t,swap)
      determ=1.0
      do 20 j=1,n
  20  ipivot(j)=0
      do 555 i=1,n
      amax=0.0
      do 105 j=1,n
      if(ipivot(j).eq.1) go to 105
      do 100 k=1,n
      if(ipivot(k)-1) 80,100,740
  80  if(abs(amax).ge.abs(a(j,k))) go to 100
      irow=j
      icolum=k
      amax=a(j,k)
 100  continue
 105  continue
      ipivot(icolum)=ipivot(icolum)+1
      if(irow.eq.icolum) go to 260
      determ=-determ
      do 200 l=1,n
      swap=a(irow,l)
      a(irow,l)=a(icolum,l)
 200  a(icolum,l)=swap
      if(m.eq.0) go to 260
      swap=b(irow)
      b(irow)=b(icolum)
      b(icolum)=swap
 260  index(i,1)=irow
      index(i,2)=icolum
      pivot(i)=a(icolum,icolum)
      determ=determ*pivot(i)
      a(icolum,icolum)=1.0
      do 350 l=1,n
 350  a(icolum,l)=a(icolum,l)/pivot(i)
      if(m.eq.0) go to 380
      b(icolum)=b(icolum)/pivot(i)
 380  do 550 l1=1,n
      if(l1.eq.icolum) go to 550
      t=a(l1,icolum)
      a(l1,icolum)=0.0
      do 450 l=1,n
 450  a(l1,l)=a(l1,l)-a(icolum,l)*t
      if(m.eq.0) go to 550
      b(l1)=b(l1)-b(icolum)*t
 550  continue
 555  continue
      do 710 i=1,n
      l=n+1-i
      if(index(l,1).eq.index(l,2)) go to 710
      jrow=index(l,1)
      jcolum=index(l,2)
      do 700  k=1,n
      swap=a(k,jrow)
      a(k,jrow)=a(k,jcolum)
 700  a(k,jcolum)=swap
 710  continue
 740  continue
      return
      end

      subroutine btobij(na,b,isub)
      dimension b(*),isub(*),bij(6)
      common /ceat/ eat(6,3)
      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      dimension j1(6),j2(6)
      data j1/1,2,3,2,3,1/, j2/1,2,3,3,1,2/
c     eat is used in rdintl(),rdsymw(),egvctr()
      do 100 n=1,nsub
          do 200 j=1,6
          k=j1(j)
          l=j2(j)
          eat(j,n)=gabn(k,l,n)/4
 200      continue
 100  continue
      write(6,*) ' equivalent b11,b22,b33,b23,b31,b12'
      do 10 i=1,na
          js=isub(i)
          do 11 j=1,6
          bij(j)=eat(j,js)*b(i)
 11       continue
          write(6,'(1h ,a,i3,6f9.5)') 'atom',i,(bij(j),j=1,6)
 10   continue
      return 
      end

      subroutine setxbp(x0,b0,p0,na,it)
      real x0(6,*),b0(it,*),p0(*)
      common /cxbp/ xx(6,100),bb(6,100),pp(100)
      if(na.gt.100) call quit('na > 100 in setxbp')
      do i=1,na
         do j=1,6
            xx(j,i)=x0(j,i)
         end do
         do j=1,it
            bb(j,i)=b0(j,i)
         end do
         pp(i)=p0(i)
c        write(6,*) 'xbp=',(xx(j,i),j=1,3),bb(1,i),pp(i)
      end do
      return
      end

      subroutine getxbp(x,b,p,na,it)
      real x(6,*),b(6,*),p(*)
      common /cxbp/ xx(6,100),bb(6,100),pp(100)
      if(na.gt.100) call quit('na > 100 in getxbp')
      do i=1,na
         do j=1,6
            x(j,i)=xx(j,i)
         end do
         do j=1,it
            b(j,i)=bb(j,i)
         end do
         p(i)=pp(i)
      end do
      return
      end
